@@ -22,9 +22,6 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <errno.h>
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
 #include <sys/stat.h>
 #include <string.h>
 #include <signal.h>
@@ -32,9 +29,10 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_UNICODE
-#include <unicode.h>
-#endif 
+
+#ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED /* gtk_signal_connect[_object] */
+#endif
 
 #include <gtk/gtk.h>
 #include <gmodule.h>
@@ -51,10 +49,8 @@
 #endif
 #endif
 
-#include <parser.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#include <xmlerror.h>
-#endif
+#include <libxml/parser.h>
+#include <libxml/xmlerror.h>
 
 #ifdef G_OS_WIN32
 #include <direct.h>
@@ -79,6 +75,10 @@
 #include "sheet.h"
 #include "plug-ins.h"
 #include "recent_files.h"
+#include "authors.h"
+#include "autosave.h"
+#include "dynamic_refresh.h"
+#include "dia_image.h"
 
 #if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
 extern DiaExportFilter png_export_filter;
@@ -113,7 +113,7 @@ save_state (GnomeClient        *client,
 
   argv[i++] = "dia";
 
-  for(l = open_diagrams; l != NULL; l = g_list_next(l)) {
+  for(l = dia_open_diagrams(); l != NULL; l = g_list_next(l)) {
     dia = (Diagram *)l->data;
     if(!dia->unsaved) {
       argv[i++] = dia->filename;
@@ -162,7 +162,7 @@ do_convert(const char *infname,
     exit(1);
   }
   
-  diagdata = new_diagram_data();
+  diagdata = new_diagram_data(&prefs.new_diagram);
   inf = filter_guess_import_filter(infname);
   if (!inf) 
     inf = &dia_import_filter;
@@ -188,10 +188,12 @@ do_convert(const char *infname,
 }
 
 void debug_break(void); /* shut gcc up */
+int debug_break_dont_optimize = 1;
 void
 debug_break(void)
 {
-  /* Break here. All symbols are loaded. */
+  if (debug_break_dont_optimize > 0) 
+    debug_break_dont_optimize -= 1;
 }
 
 gboolean
@@ -220,6 +222,8 @@ void
 app_init (int argc, char **argv)
 {
   gboolean nosplash = FALSE;
+  gboolean credits = FALSE;
+  gboolean version = FALSE;
 #ifdef GNOME
   GnomeClient *client;
 #endif
@@ -240,6 +244,10 @@ app_init (int argc, char **argv)
      0, N_("Export to file format and exit"), N_("eps,png,wmf,cgm,dxf,fig")},
     {"nosplash", 'n', POPT_ARG_NONE, &nosplash, 0,
      N_("Don't show the splash screen"), NULL },
+    {"credits", 'c', POPT_ARG_NONE, &credits, 0,
+     N_("Display credits list and exit"), NULL },
+    {"version", 'v', POPT_ARG_NONE, &version, 0,
+     N_("Display version and exit"), NULL },
 #ifndef GNOME
     {"help", 'h', POPT_ARG_NONE, 0, 1, N_("Show this help message") },
 #endif
@@ -257,21 +265,19 @@ app_init (int argc, char **argv)
   gtk_set_locale();
   setlocale(LC_NUMERIC, "C");
   
-#ifdef HAVE_UNICODE
-  unicode_init();
-#endif 
-
-  bindtextdomain(PACKAGE, LOCALEDIR);
-#if defined GTK_TALKS_UTF8 && defined ENABLE_NLS
-  bind_textdomain_codeset(PACKAGE,"UTF-8");  
+  bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
+#if defined ENABLE_NLS
+  bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");  
 #endif
-  textdomain(PACKAGE);
+  textdomain(GETTEXT_PACKAGE);
 
   if (argv) {
 #ifdef GNOME
-    gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv, options,
-			       0, &poptCtx);
-    
+    GnomeProgram *program =
+      gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
+			  argc, argv, GNOME_PARAM_POPT_TABLE, options,
+			  GNOME_PARAM_NONE);
+    g_object_get(program, "popt-context", &poptCtx, NULL);
     client = gnome_master_client();
     if(client == NULL) {
       g_warning(_("Can't connect to session manager!\n"));
@@ -282,7 +288,11 @@ app_init (int argc, char **argv)
       gtk_signal_connect(GTK_OBJECT (client), "die",
 			 GTK_SIGNAL_FUNC (session_die), NULL);
     }
+
+    /* This smaller icon is 48x48, standard Gnome size */
+    gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/dia_gnome_icon.png");
 #else
+    gtk_init (&argc, &argv);
 #ifdef HAVE_POPT
     poptCtx = poptGetContext(PACKAGE, argc, (const char **)argv, options, 0);
     poptSetOtherOptionHelp(poptCtx, _("[OPTION...] [FILE...]"));
@@ -309,10 +319,57 @@ app_init (int argc, char **argv)
 
     }
 #endif
-    gtk_init (&argc, &argv);
 #endif
   }
 
+  if (version) {
+#if (defined __TIME__) && (defined __DATE__)
+    /* TRANSLATOR: 2nd and 3rd %s are time and date respectively. */
+    printf(_("Dia version %s, compiled %s %s\n"), VERSION, __TIME__, __DATE__);
+#else
+    printf(_("Dia version %s\n"), VERSION);
+#endif
+    exit(0);
+  }
+
+  /* --credits option. Added by Andrew Ferrier.
+  
+     Hopefully we're not ignoring anything too crucial by
+     quitting directly after the credits.
+
+     The phrasing of the English here might need changing
+     if we switch from plural to non-plural (say, only
+     one maintainer).
+  */
+
+  if (credits) {
+      int i;
+      const gint nauthors = (sizeof(authors) / sizeof(authors[0])) - 1;
+      const gint ndocumentors = (sizeof(documentors) / sizeof(documentors[0])) - 1;
+
+      printf("The original author of Dia was:\n\n");
+      for (i = 0; i < NUMBER_OF_ORIG_AUTHORS; i++) {
+          printf(authors[i]); printf("\n");
+      }
+
+      printf("\nThe current maintainers of Dia are:\n\n");
+      for (i = NUMBER_OF_ORIG_AUTHORS; i < NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i++) {
+	  printf(authors[i]); printf("\n");
+      }
+
+      printf("\nOther authors are:\n\n");
+      for (i = NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i < nauthors; i++) {
+          printf(authors[i]); printf("\n");
+      }
+
+      printf("\nDia is documented by:\n\n");
+      for (i = 0; i < ndocumentors; i++) {
+          printf(documentors[i]); printf("\n");
+      }
+
+      exit(0);
+  }
+
   LIBXML_TEST_VERSION;
 
 #ifdef G_OS_WIN32
@@ -323,6 +380,7 @@ app_init (int argc, char **argv)
 
   dia_image_init();
 
+  
   gdk_rgb_init();
 
   gtk_rc_parse ("diagtkrc"); 
@@ -334,7 +392,11 @@ app_init (int argc, char **argv)
 
   color_init();
 
-  font_init();
+#ifdef HAVE_FREETYPE
+  dia_font_init(pango_ft2_get_context(10,10));
+#else
+  dia_font_init(gdk_pango_context_get());
+#endif
 
   /* Init cursors: */
   default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
@@ -347,6 +409,8 @@ app_init (int argc, char **argv)
 
   load_all_sheets();     /* new mechanism */
 
+  dia_object_defaults_load (NULL, TRUE /* prefs.object_defaults_create_lazy */);
+
   debug_break();
 
   if (object_get_type("Standard - Box") == NULL) {
@@ -369,10 +433,14 @@ app_init (int argc, char **argv)
   /*fill recent file menu */
   recent_file_history_init();
 
+  /* Set up autosave to check every 5 minutes */
+  gtk_timeout_add(5*60*1000, autosave_check_autosave, NULL);
+
   create_tree_window();
 
+  /* In current setup, we can't find the autosaved files. */
   /*autosave_restore_documents();*/
-
+  
   if (argv) {
 #ifdef HAVE_POPT
       while (poptPeekArg(poptCtx)) {
@@ -442,6 +510,8 @@ app_init (int argc, char **argv)
 #endif
   }
   if (made_conversions) exit(0);
+
+  dynobj_refresh_init();
 }
 
 static void
@@ -455,7 +525,7 @@ app_exit(void)
 {
   GList *list;
   GSList *slist;
-  gchar *filename;
+
   /*
    * The following "solves" a crash related to a second call of app_exit,
    * after gtk_main_quit was called. It may be a win32 gtk-1.3.x bug only
@@ -464,96 +534,45 @@ app_exit(void)
   static gboolean app_exit_once = FALSE;
 
   g_return_if_fail (!app_exit_once);
-  
+
   if (diagram_modified_exists()) {
     GtkWidget *dialog;
-    GtkWidget *vbox;
-    GtkWidget *label;
-#ifndef GNOME
     GtkWidget *button;
-#endif
-    int result = FALSE;
 
-#ifdef GNOME
-    dialog = gnome_dialog_new(_("Quit, are you sure?"), NULL);
-    vbox = GNOME_DIALOG(dialog)->vbox;
-    gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
-#else
-    dialog = gtk_dialog_new();
-    vbox = GTK_DIALOG(dialog)->vbox;
-    gtk_window_set_title (GTK_WINDOW (dialog), _("Quit, are you sure?"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-#endif
+    dialog = gtk_message_dialog_new(
+	       NULL, GTK_DIALOG_MODAL,
+               GTK_MESSAGE_QUESTION,
+               GTK_BUTTONS_NONE, /* no standard buttons */
+	       _("Modified diagrams exist.\n"
+           "Are you sure you want to quit Dia\n"
+           "without saving them?"),
+	       NULL);
+    gtk_window_set_title (GTK_WINDOW(dialog), _("Quit Dia"));
 
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    label = gtk_label_new (_("Modified diagrams exists.\n"
-			   "Are you sure you want to quit?"));
-  
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-  
-    gtk_widget_show (label);
-
-#ifdef GNOME
-    gnome_dialog_append_button_with_pixmap(GNOME_DIALOG(dialog),
-					   _("Quit"), GNOME_STOCK_PIXMAP_QUIT);
-    gnome_dialog_append_button(GNOME_DIALOG(dialog),GNOME_STOCK_BUTTON_CANCEL);
-
-    result = (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0);
-#else
-    button = gtk_button_new_with_label (_("Quit"));
+    button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+    gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, GTK_RESPONSE_CANCEL);
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("Cancel"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    
-    gtk_widget_show (button);
+    gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
 
-    gtk_widget_show (dialog);
+    button = gtk_button_new_from_stock (GTK_STOCK_QUIT);
+    gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, GTK_RESPONSE_OK);
 
-    /* Make dialog modal: */
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
+    g_signal_connect_after (G_OBJECT (dialog), "response",
+		            G_CALLBACK(gtk_widget_destroy),
+		            NULL);
+    gtk_widget_show_all (dialog);
 
-    gtk_main();
-#endif
-
-    if (result == FALSE)
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK)
       return;
   }
-  
-  /* Save menu accelerators */
-  filename = dia_config_filename("menus" G_DIR_SEPARATOR_S "toolbox");
-  if (filename!=NULL) {
-    GtkPatternSpec pattern;
 
-    gtk_pattern_spec_init(&pattern, "*<Toolbox>*");
-
-    gtk_item_factory_dump_rc (filename, &pattern, TRUE);
-    g_free (filename);
-    gtk_pattern_spec_free_segs(&pattern);
-  }
+  dynobj_refresh_finish();
+    
+  dia_object_defaults_save (NULL);
 
   /* Free loads of stuff (toolbox) */
 
-  list = open_diagrams;
+  list = dia_open_diagrams();
   while (list!=NULL) {
     Diagram *dia = (Diagram *)list->data;
     list = g_list_next(list);
