@@ -1,6 +1,5 @@
 /* Dia -- an diagram creation/manipulation program
- * Copyright (C) 1998 Alexander Larsson
- *
+ * Copyright (C) 1998 Alexander Larsson *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -28,11 +27,15 @@
 #include <sys/stat.h>
 #include <math.h>
 #include <glib.h>
-#ifdef HAVE_DIRENT_H
-#  include <dirent.h>
-#endif
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtk.h>
+
+/* Added by Andrew Ferrier: for use of gnome_about_new() */
+
+#ifdef GNOME
+#  include <gnome.h>
+#endif
 
 #ifdef G_OS_WIN32
 /*
@@ -49,16 +52,9 @@ ShellExecuteA (long        hwnd,
                int         nShowCmd);
 #endif
 
-#ifdef GNOME
-#include <gnome.h>
-#endif
-#ifdef GNOME_PRINT
-#include "paginate_gnomeprint.h"
-#else
 #include "paginate_psprint.h"
-#  ifdef G_OS_WIN32
+#ifdef G_OS_WIN32
 #  include "paginate_gdiprint.h"
-#  endif
 #endif
 #include "intl.h"
 #include "commands.h"
@@ -86,6 +82,7 @@ ShellExecuteA (long        hwnd,
 #include "lib/properties.h"
 #include "dia-props.h"
 #include "diagram_tree_window.h"
+#include "authors.h"                /* master contributors data */
 
 GdkPixbuf *logo;
 
@@ -109,17 +106,13 @@ file_print_callback(gpointer data, guint action, GtkWidget *widget)
   Diagram *dia;
 
   dia = ddisplay_active()->diagram;
-#ifdef GNOME_PRINT
-  diagram_print_gnome(dia);
-#else
-#  ifdef G_OS_WIN32
+#ifdef G_OS_WIN32
   /* This option could be used with Gnome too. Does it make sense there ? */
   if (!prefs.prefer_psprint)
     diagram_print_gdi(dia);
   else
-#  endif
-    diagram_print_ps(dia);
 #endif
+    diagram_print_ps(dia);
 }
 
 void
@@ -136,7 +129,7 @@ file_new_callback(gpointer data, guint action, GtkWidget *widget)
   static int untitled_nr = 1;
   char buffer[24];
 
-  g_snprintf(buffer, 24, _("Untitled-%d"), untitled_nr++);
+  g_snprintf(buffer, 24, _("Diagram%d.dia"), untitled_nr++);
   
   dia = new_diagram(buffer);
   ddisp = new_display(dia);
@@ -160,6 +153,7 @@ edit_copy_callback(gpointer data, guint action, GtkWidget *widget)
   copy_list = diagram_get_sorted_selected(ddisp->diagram);
 
   cnp_store_objects(object_copy_list(copy_list));
+  g_list_free(copy_list);
 
   ddisplay_do_update_menu_sensitivity(ddisp);
 }
@@ -230,42 +224,30 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
   diagram_flush(ddisp->diagram);
 }
 
-/* Signal handler for getting the selection from whoever */
-void
-received_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
-			   gpointer data)
+/* Signal handler for getting the clipboard contents */
+/* Note that the clipboard is for M$-style cut/copy/paste copying, while
+   the selection is for Unix-style mark-and-copy.  We can't really do
+   mark-and-copy.
+*/
+
+static void
+insert_text(DDisplay *ddisp, Focus *focus, const gchar *text)
 {
-  Focus *focus = active_focus();
-  gchar *text;
-  int i;
   ObjectChange *change = NULL;
-  int modified, any_modified = FALSE;
-  Object *obj;
-  DDisplay *ddisp = ddisplay_active();
+  int modified = FALSE, any_modified = FALSE;
+  Object *obj = focus->obj;
 
-  if ((focus == NULL) || (!focus->has_focus)) return;
-
-  obj = focus->obj;
-
-  /* **** IMPORTANT **** Check to see if retrieval succeeded  */
-  if (selection->length < 0)
-  {
-    g_print ("Selection retrieval failed\n");
-    return;
-  }
-  /* Make sure we got the data in the expected form */
-  if (selection->type != GDK_SELECTION_TYPE_STRING)
-  {
-    g_print ("Selection \"STRING\" was not returned as a string!\n");
-    return;
-  }
-
-  text = (gchar *)selection->data;
-  for (i = 0; i < selection->length; i++) {
-    if (text[i] == '\n') {
+  while (text != NULL) {
+    gchar *next_line = g_utf8_strchr(text, -1, '\n');
+    if (next_line != text) {
+      gint len = g_utf8_strlen(text, (next_line-text));
+      modified = (*focus->key_event)(focus, GDK_A, text, len, &change);
+    }
+    if (next_line != NULL) {
       modified = (*focus->key_event)(focus, GDK_Return, "\n", 1, &change);
+      text = g_utf8_next_char(next_line);
     } else {
-      modified = (*focus->key_event)(focus, GDK_A, &text[i], 1, &change);
+      text = NULL;
     }
     { /* Make sure object updates its data: */
       Point p = obj->position;
@@ -286,33 +268,39 @@ received_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
     undo_set_transactionpoint(ddisp->diagram->undo);
 }
 
-static char *current_clipboard;
 
-void
-get_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
-		      gpointer data) {
-  if (current_clipboard) {
-    gtk_selection_data_set(selection, GDK_TARGET_STRING,
-			   8, current_clipboard, strlen(current_clipboard));
+static void
+received_clipboard_handler(GtkClipboard *clipboard, 
+			   const gchar *text,
+			   gpointer data) {
+  Focus *focus = active_focus();
+  DDisplay *ddisp = (DDisplay *)data;
+  
+  if (text == NULL) return;
+
+  if ((focus == NULL) || (!focus->has_focus)) return;
+
+  if (!g_utf8_validate(text, -1, NULL)) {
+    message_error("Not valid UTF8");
+    return;
   }
+
+  insert_text(ddisp, focus, text);
 }
 
-
 static PropDescription text_prop_singleton_desc[] = {
-    { "text", PROP_TYPE_STRING },
+    { "text", PROP_TYPE_TEXT },
     PROP_DESC_END};
 
-
 static void 
-make_text_prop_singleton(GPtrArray **props, StringProperty **prop) 
+make_text_prop_singleton(GPtrArray **props, TextProperty **prop) 
 {
   *props = prop_list_from_descs(text_prop_singleton_desc,pdtpp_true);
   g_assert((*props)->len == 1);
 
   *prop = g_ptr_array_index((*props),0);
-  g_free((*prop)->string_data);
-  (*prop)->string_data = NULL;
-  (*prop)->num_lines = 0;
+  g_free((*prop)->text_data);
+  (*prop)->text_data = NULL;
 }
 
 
@@ -323,7 +311,7 @@ edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
   DDisplay *ddisp;
   Object *obj;
   GPtrArray *textprops;
-  StringProperty *prop;
+  TextProperty *prop;
 
   if ((focus == NULL) || (!focus->has_focus)) return;
 
@@ -338,18 +326,17 @@ edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
   /* Get the first text property */
   obj->ops->get_props(obj, textprops);
   
-  if (current_clipboard) g_free(current_clipboard);
-
-  current_clipboard = g_strdup(prop->string_data);
-
+  /* GTK docs claim the selection clipboard is ignored on Win32.
+   * The "clipboard" clipboard is mostly ignored in Unix
+   */
+#ifdef G_OS_WIN32
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE),
+			 prop->text_data, -1);
+#else
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+			 prop->text_data, -1);
+#endif
   prop_list_free(textprops);
-
-  gtk_selection_owner_set(GTK_WIDGET(ddisp->shell),
-			  GDK_SELECTION_PRIMARY,
-			  GDK_CURRENT_TIME);
-  gtk_selection_add_target(GTK_WIDGET(ddisp->shell),
-			   GDK_SELECTION_PRIMARY,
-			   GDK_TARGET_STRING, 0);
 }
 
 void
@@ -360,15 +347,15 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   Object *obj;
   Text *text;
   GPtrArray *textprops;
-  StringProperty *prop;
-  Change *change;
+  TextProperty *prop;
+  ObjectChange *change;
 
   if ((focus == NULL) || (!focus->has_focus)) return;
 
   ddisp = ddisplay_active();
 
   obj = focus->obj;
-  text = (Text *)obj; 
+  text = (Text *)focus->user_data; 
 
   if (obj->ops->get_props == NULL) 
     return;
@@ -377,23 +364,23 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   /* Get the first text property */
   obj->ops->get_props(obj, textprops);
   
-  if (current_clipboard) g_free(current_clipboard);
-
-  current_clipboard = g_strdup(prop->string_data);
+  /* GTK docs claim the selection clipboard is ignored on Win32.
+   * The "clipboard" clipboard is mostly ignored in Unix
+   */
+#ifdef G_OS_WIN32
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE),
+			 prop->text_data, -1);
+#else
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+			 prop->text_data, -1);
+#endif
 
   prop_list_free(textprops);
 
-  gtk_selection_owner_set(GTK_WIDGET(ddisp->shell),
-			  GDK_SELECTION_PRIMARY,
-			  GDK_CURRENT_TIME);
-  gtk_selection_add_target(GTK_WIDGET(ddisp->shell),
-			   GDK_SELECTION_PRIMARY,
-			   GDK_TARGET_STRING, 0);
-
-  if (text_delete_all(text, (ObjectChange **)&change)) { 
-    (change->apply)(change, ddisp->diagram);
-    
-    diagram_update_menu_sensitivity(ddisp->diagram, &ddisp->updatable_menu_items);
+  if (text_delete_all(text, &change)) { 
+    object_add_updates(obj, ddisp->diagram);
+    undo_object_change(ddisp->diagram, obj, change);
+    undo_set_transactionpoint(ddisp->diagram->undo);
     diagram_flush(ddisp->diagram);
   }
 }
@@ -405,8 +392,13 @@ edit_paste_text_callback(gpointer data, guint action, GtkWidget *widget)
 
   ddisp = ddisplay_active();
 
-  gtk_selection_convert(ddisp->shell, GDK_SELECTION_PRIMARY, GDK_TARGET_STRING,
-			GDK_CURRENT_TIME);
+#ifdef G_OS_WIN32
+  gtk_clipboard_request_text(gtk_clipboard_get(GDK_NONE), 
+			     received_clipboard_handler, ddisp);
+#else
+  gtk_clipboard_request_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY), 
+			     received_clipboard_handler, ddisp);
+#endif
 }
 
 void
@@ -467,8 +459,9 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
 {
   char *helpdir, *helpindex = NULL, *command;
   guint bestscore = G_MAXINT;
-  DIR *dp;
-  struct dirent *dirp;
+  GDir *dp;
+  const char *dentry;
+  GError *error = NULL;
 
   helpdir = dia_get_data_directory("help");
   if (!helpdir) {
@@ -477,29 +470,27 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
   }
 
   /* search through helpdir for the helpfile that matches the user's locale */
-  dp = opendir(helpdir);
-  if (dp == NULL) {
-    g_free(helpdir);
-    message_warning(_("Could not find help directory"));
+  dp = g_dir_open (helpdir, 0, &error);
+  if (!dp) {
+    message_warning(_("Could not open help directory:\n%s"),
+                    error->message);
+    g_error_free (error);
     return;
   }
-
-  while ((dirp = readdir(dp)) != NULL) {
+  
+  while ((dentry = g_dir_read_name(dp)) != NULL) {
     guint score;
 
-    if (!strcmp(dirp->d_name, ".") || !strcmp(dirp->d_name, ".."))
-      continue;
-
-    score = intl_score_locale(dirp->d_name);
+    score = intl_score_locale(dentry);
     if (score < bestscore) {
       if (helpindex)
 	g_free(helpindex);
-      helpindex = g_strconcat(helpdir, G_DIR_SEPARATOR_S, dirp->d_name,
+      helpindex = g_strconcat(helpdir, G_DIR_SEPARATOR_S, dentry,
 			      G_DIR_SEPARATOR_S "index.html", NULL);
       bestscore = score;
     }
   }
-  closedir(dp);
+  g_dir_close (dp);
   g_free(helpdir);
   if (!helpindex) {
     message_warning(_("Could not find help directory"));
@@ -509,7 +500,10 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
 #ifdef G_OS_WIN32
   ShellExecuteA (0, NULL, helpindex, NULL, helpdir, 0); 
 #else
-  command = g_strdup_printf("netscape '%s' &", helpindex);
+  command = getenv("BROWSER");
+  if (command == NULL)
+    command = g_strdup_printf("netscape");
+  command = g_strdup_printf("%s 'file://%s' &", command, helpindex);
   system(command);
   g_free(command);
 #endif
@@ -520,6 +514,67 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
 void
 help_about_callback(gpointer data, guint action, GtkWidget *widget)
 {
+#ifdef GNOME
+
+  /*  Take advantage of gnome_about_new(),
+   *  which is much cleaner and GNOME2 HIG compliant,
+   *  Originally implemented by Xing Wang, modified
+   *  by Andrew Ferrier.
+   *
+   *  Note: in this function there is no need to discriminate
+   *  between the different kinds of 'authors'.
+   */
+  
+  static GtkWidget *about;
+  
+  /*
+   * Translators should localize the following string
+   * which will give them credit in the About box.
+   * E.g. "Fulano de Tal <fulano@detal.com>"
+   */
+  
+  gchar *translators = _("translator_credits-PLEASE_ADD_YOURSELF_HERE");
+  gchar logo_file[100];
+  
+  if (!about) {
+    GdkPixbuf *logo;
+  
+    gchar* datadir = dia_get_data_directory(""); 
+    g_snprintf(logo_file, sizeof(logo_file),
+                             "%s%sdia_logo.png", datadir, G_DIR_SEPARATOR_S);
+
+  	logo = gdk_pixbuf_new_from_file(logo_file, NULL);
+    g_free(datadir);
+  
+    about = gnome_about_new(
+      _("Dia"),
+      VERSION,
+      _("Copyright (C) 1998-2002 The Free Software Foundation and the authors"),
+      _("Dia is a program for drawing structured diagrams.\n"
+      "Please visit http://www.lysator.liu.se/~alla/dia for more information."),
+      authors,
+      documentors,
+      (strcmp (translators, "translator_credits-PLEASE_ADD_YOURSELF_HERE")
+      ? translators : NULL),
+      logo);
+  
+    if (logo)
+        g_object_unref (logo);
+      
+  	g_signal_connect (about, "destroy",
+         G_CALLBACK (gtk_widget_destroyed),
+         &about);
+  }
+
+  gtk_widget_show_now (about);
+  
+#else
+
+  /* No GNOME, fall back to the old GTK method */
+
+  const gint nauthors = (sizeof(authors) / sizeof(authors[0])) - 1;
+  const gint ndocumentors = (sizeof(documentors) / sizeof(documentors[0])) - 1;
+
   GtkWidget *dialog;
   GtkWidget *vbox;
   GtkWidget *table;
@@ -531,31 +586,15 @@ help_about_callback(gpointer data, guint action, GtkWidget *widget)
   gint i;
   
   GtkWidget *gpixmap;
-
-  static const gchar *contributors[] = {
-    "Jerome Abela",
-    "Hans Breuer",
-    "Emmanuel Briot",
-    "Cyrille Chepelov",
-    "Lars R. Clausen",
-    "Fredrik Hallenberg",
-    "Francis J. Lacoste",
-    "Steffen Macke",
-    "M. C. Nelson",
-    "Jacek Pliszka",
-    "Henk Jan Priester",
-    "Alejandro Aguilar Sierra",
-  };
-  const gint ncontributors = sizeof(contributors) / sizeof(contributors[0]);
-
+  
   dialog = gtk_dialog_new ();
-  gtk_window_set_wmclass (GTK_WINDOW (dialog), "about_dialog", "Dia");
+  gtk_window_set_role (GTK_WINDOW (dialog), "about_dialog");
   gtk_window_set_title (GTK_WINDOW (dialog), _("About Dia"));
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
+  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_widget_destroy), 
-		      GTK_OBJECT (dialog));
+          GTK_SIGNAL_FUNC (gtk_widget_destroy), 
+          GTK_OBJECT (dialog));
 
   vbox = gtk_vbox_new (FALSE, 1);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
@@ -564,7 +603,7 @@ help_about_callback(gpointer data, guint action, GtkWidget *widget)
   if (!logo) {
       gchar* datadir = dia_get_data_directory(""); 
       g_snprintf(str, sizeof(str), "%s%sdia_logo.png", datadir, G_DIR_SEPARATOR_S);
-      logo = gdk_pixbuf_new_from_file(str);
+      logo = gdk_pixbuf_new_from_file(str, NULL);
       g_free(datadir);
   }
 
@@ -596,26 +635,34 @@ help_about_callback(gpointer data, guint action, GtkWidget *widget)
   g_snprintf(str, sizeof(str), _("Dia v %s by Alexander Larsson"), VERSION);
   label = gtk_label_new (str);
   gtk_table_attach(GTK_TABLE(table), label, 0,2, 0,1,
-		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
-  
-  label = gtk_label_new(_("Maintainer: James Henstridge"));
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+
+      /* Exact spelling is Ch&eacute;p&eacute;lov (using *ML entities) */
+  label = gtk_label_new(_("Maintainers: Lars Clausen and Cyrille Chepelov"));
   gtk_table_attach(GTK_TABLE(table), label, 0,2, 1,2,
-		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
 
   label = gtk_label_new (_("Please visit http://www.lysator.liu.se/~alla/dia "
-			   "for more info"));
+         "for more information"));
   gtk_table_attach(GTK_TABLE(table), label, 0,2, 2,3,
-		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
 
   label = gtk_label_new (_("Contributors:"));
   gtk_table_attach(GTK_TABLE(table), label, 0,2, 3,4,
-		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
 
-  for (i = 0; i < ncontributors; i++) {
-    label = gtk_label_new(contributors[i]);
+  for (i = 0; i < nauthors; i++) {
+    label = gtk_label_new(authors[i]);
     gtk_table_attach(GTK_TABLE(table), label, i%2,i%2+1, i/2+4,i/2+5,
-		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
   }
+
+  for (i = nauthors; i < nauthors + ndocumentors; i++) {
+    label = gtk_label_new(documentors[i - nauthors]);
+    gtk_table_attach(GTK_TABLE(table), label, i%2,i%2+1, i/2+4,i/2+5,
+       GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
+  }
+
   gtk_table_set_col_spacings(GTK_TABLE(table), 1);
   gtk_table_set_row_spacings(GTK_TABLE(table), 1);
 
@@ -624,13 +671,15 @@ help_about_callback(gpointer data, guint action, GtkWidget *widget)
   gtk_button_box_set_child_size(GTK_BUTTON_BOX(bbox), 80, 0);
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 10);
 
-  button = gtk_button_new_with_label(_("OK"));
+  button = gtk_button_new_from_stock(GTK_STOCK_OK);
   gtk_container_add(GTK_CONTAINER(bbox), button);
   gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			    GTK_OBJECT(dialog));
+          GTK_SIGNAL_FUNC(gtk_widget_destroy),
+          GTK_OBJECT(dialog));
 
   gtk_widget_show_all (dialog);
+
+#endif /*  GNOME  */
 }
 
 void
@@ -703,18 +752,11 @@ view_aa_callback(gpointer data, guint action, GtkWidget *widget)
 {
   DDisplay *ddisp;
   int aa;
-  static gboolean shown_warning = FALSE;
 
   ddisp = ddisplay_active();
  
   aa =  GTK_CHECK_MENU_ITEM(widget)->active;
   
-  if (aa && !shown_warning)
-    message_warning(_("The anti aliased renderer is buggy, and may cause\n"
-		      "crashes.  We know there are bugs in it, so don't\n"
-		      "bother submitting another report if it crashes"));
-  shown_warning = TRUE;
-
   if (aa != ddisp->aa_renderer) {
     ddisplay_set_renderer(ddisp, aa);
     ddisplay_add_update_all(ddisp);
@@ -815,8 +857,8 @@ view_show_all_callback(gpointer data, guint action, GtkWidget *widget)
   ddisp = ddisplay_active();
   dia = ddisp->diagram;
 
-  width = ddisp->renderer->pixel_width;
-  height = ddisp->renderer->pixel_height;
+  width = dia_renderer_get_width_pixels (ddisp->renderer);
+  height = dia_renderer_get_height_pixels (ddisp->renderer);
 
   magnify_x = (real)width /
     (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
@@ -835,6 +877,15 @@ view_show_all_callback(gpointer data, guint action, GtkWidget *widget)
   ddisplay_flush(ddisp);
 }
 
+void
+view_redraw_callback(gpointer data, guint action, GtkWidget *widget)
+{
+  DDisplay *ddisp;
+  ddisp = ddisplay_active();
+  ddisplay_add_update_all(ddisp);
+  ddisplay_flush(ddisp);
+}
+
 void
 view_diagram_properties_callback(gpointer data, guint action, GtkWidget *widget)
 {
@@ -857,6 +908,18 @@ objects_place_under_callback(gpointer data, guint action, GtkWidget *widget)
   diagram_place_under_selected(ddisplay_active()->diagram);
 }
 
+void
+objects_place_up_callback(gpointer data, guint action, GtkWidget *widget)
+{
+  diagram_place_up_selected(ddisplay_active()->diagram);
+}
+
+void
+objects_place_down_callback(gpointer data, guint action, GtkWidget *widget)
+{
+  diagram_place_down_selected(ddisplay_active()->diagram);
+}
+
 void
 objects_group_callback(gpointer data, guint action, GtkWidget *widget)
 {
@@ -878,11 +941,11 @@ dialogs_properties_callback(gpointer data, guint action, GtkWidget *widget)
   dia = ddisplay_active()->diagram; 
 
   if (dia->data->selected != NULL) {
-       selected = dia->data->selected->data;
+    selected = dia->data->selected->data;
+    properties_show(dia, selected);
   } else {
-         selected = NULL;
+    diagram_properties_show(dia);
   } 
-  properties_show(dia, selected);
 }
 
 void
