@@ -38,8 +38,16 @@
 #include "load_save.h"
 #include "recent_files.h"
 #include "diagram_tree_window.h"
+#include "autosave.h"
+#include "dynamic_refresh.h"
 
-GList *open_diagrams = NULL;
+static GList *open_diagrams = NULL;
+
+GList *
+dia_open_diagrams(void)
+{
+  return open_diagrams;
+}
 
 static void
 diagram_init(Diagram *dia, const char *filename)
@@ -47,15 +55,17 @@ diagram_init(Diagram *dia, const char *filename)
   if(dia->data)
     diagram_data_destroy(dia->data);
   
-  dia->data = new_diagram_data();
+  dia->data = new_diagram_data(&prefs.new_diagram);
   dia->data->grid.width_x = prefs.grid.x;
   dia->data->grid.width_y = prefs.grid.y;
 
-  g_free(dia->filename);
+  if (dia->filename != NULL)
+    g_free(dia->filename);
   dia->filename = g_strdup(filename);
   
   dia->unsaved = TRUE;
   dia->modified = FALSE;
+  dia->autosavefilename = NULL;
 
   if (dia->undo)
     undo_destroy(dia->undo);
@@ -109,9 +119,6 @@ new_diagram(const char *filename)  /* Note: filename is copied */
 {
   Diagram *dia = g_new0(Diagram, 1);
 
-  dia->display_count = 0;
-  dia->displays = NULL;
-
   diagram_init(dia, filename);
 
   return dia;
@@ -120,33 +127,20 @@ new_diagram(const char *filename)  /* Note: filename is copied */
 void
 diagram_destroy(Diagram *dia)
 {
-  char *home_path;
-
   assert(dia->displays==NULL);
 
   diagram_data_destroy(dia->data);
   
   g_free(dia->filename);
 
-  /* Save menu accelerators */
-  home_path = dia_config_filename("menus" G_DIR_SEPARATOR_S "display");
-
-  if (home_path != NULL) {
-    GtkPatternSpec pattern;
-
-    gtk_pattern_spec_init(&pattern, "*<Display>*");
-
-    gtk_item_factory_dump_rc (home_path, &pattern, TRUE);
-    g_free (home_path);
-    gtk_pattern_spec_free_segs(&pattern);
-  }
-
   open_diagrams = g_list_remove(open_diagrams, dia);
   layer_dialog_update_diagram_list();
 
   undo_destroy(dia->undo);
   
   diagram_tree_remove(diagram_tree(), dia);
+
+  diagram_cleanup_autosave(dia);
   
   g_free(dia);
 }
@@ -184,35 +178,54 @@ diagram_set_modified(Diagram *dia, int modified)
 void 
 diagram_update_menu_sensitivity (Diagram *dia, UpdatableMenuItems *items)
 {
-  gtk_widget_set_sensitive(items->copy, dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(items->cut, dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(items->paste, cnp_exist_stored_objects());
+  gtk_widget_set_sensitive(GTK_WIDGET(items->copy),
+			   dia->data->selected_count > 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->cut),
+			   dia->data->selected_count > 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->paste),
+			   cnp_exist_stored_objects());
   #ifndef GNOME
-  gtk_widget_set_sensitive(items->edit_delete, dia->data->selected_count > 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->edit_delete),
+			   dia->data->selected_count > 0);
   #endif
     
-  gtk_widget_set_sensitive(items->copy_text, active_focus() != NULL);
-  gtk_widget_set_sensitive(items->cut_text, 0); /* Not implemented */
-  gtk_widget_set_sensitive(items->paste_text, active_focus() != NULL);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->copy_text),
+			   active_focus() != NULL);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->cut_text),
+			   active_focus() != NULL);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->paste_text),
+			   active_focus() != NULL);
   
-  gtk_widget_set_sensitive(items->send_to_back, dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(items->bring_to_front, dia->data->selected_count > 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->send_to_back),
+			   dia->data->selected_count > 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->bring_to_front),
+			   dia->data->selected_count > 0);
     
-  gtk_widget_set_sensitive(items->group, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->ungroup,
+  gtk_widget_set_sensitive(GTK_WIDGET(items->group),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->ungroup),
 			   (dia->data->selected_count > 0));
   
-  gtk_widget_set_sensitive(items->align_h_l, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_h_c, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_h_r, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_h_e, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_h_a, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_v_t, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_v_c, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_v_b, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_v_e, dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(items->align_v_a, dia->data->selected_count > 1);
-  
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_l),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_c),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_r),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_e),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_a),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_t),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_c),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_b),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_e),
+			   dia->data->selected_count > 1);
+  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_a),
+			   dia->data->selected_count > 1);
 }
     
   
@@ -492,6 +505,7 @@ diagram_flush(Diagram *dia)
     
     l = g_slist_next(l);
   }
+  dynobj_refresh_kick();
 }
 
 Object *
@@ -543,10 +557,11 @@ diagram_find_closest_handle(Diagram *dia, Handle **closest,
 real
 diagram_find_closest_connectionpoint(Diagram *dia,
 				     ConnectionPoint **closest,
-				     Point *pos)
+				     Point *pos,
+				     Object *notthis)
 {
   return layer_find_closest_connectionpoint(dia->data->active_layer,
-					    closest, pos);
+					    closest, pos, notthis);
 }
 
 void
@@ -752,6 +767,96 @@ diagram_place_over_selected(Diagram *dia)
   undo_set_transactionpoint(dia->undo);
 }
 
+void
+diagram_place_up_selected(Diagram *dia)
+{
+  GList *sorted_list;
+  GList *orig_list;
+  GList *tmp, *stmp;
+  GList *new_list = NULL;
+
+  if (dia->data->selected_count == 0)
+    return;
+
+  orig_list = g_list_copy(dia->data->active_layer->objects);
+  
+  sorted_list = diagram_get_sorted_selected(dia);
+  object_add_updates_list(orig_list, dia);
+
+  new_list = g_list_copy(orig_list);
+  stmp = g_list_last(sorted_list);
+
+  for (tmp = g_list_last(new_list);
+       tmp != NULL;
+       tmp = g_list_previous(tmp)) {
+    if (stmp == NULL) break;
+    if (tmp->prev == NULL) break;
+    if (tmp->data == stmp->data) {
+      stmp = g_list_previous(stmp);
+    } else if (tmp->prev->data == stmp->data) {
+      void *swap = tmp->data;
+      tmp->data = tmp->prev->data;
+      tmp->prev->data = swap;
+      stmp = g_list_previous(stmp);
+    }
+  }
+
+  layer_set_object_list(dia->data->active_layer, new_list);
+
+  undo_reorder_objects(dia, g_list_copy(sorted_list), orig_list);
+
+  diagram_modified(dia);
+  diagram_flush(dia);
+  undo_set_transactionpoint(dia->undo);
+}
+
+void
+diagram_place_down_selected(Diagram *dia)
+{
+  GList *sorted_list;
+  GList *orig_list;
+  GList *tmp, *stmp;
+  GList *new_list = NULL;
+
+  if (dia->data->selected_count == 0)
+    return;
+
+  orig_list = g_list_copy(dia->data->active_layer->objects);
+  
+  sorted_list = diagram_get_sorted_selected(dia);
+  object_add_updates_list(orig_list, dia);
+
+  /* Sanity check */
+  g_assert(dia->data->selected_count == g_list_length(sorted_list));
+
+  new_list = g_list_copy(orig_list);
+  tmp = new_list;
+  stmp = sorted_list;
+
+  for (tmp = new_list; tmp != NULL; tmp = g_list_next(tmp)) {
+    if (stmp == NULL) break;
+    if (tmp->next == NULL) break;
+    if (tmp->data == stmp->data) {
+      /* This just takes care of any starting matches */
+      stmp = g_list_next(stmp);
+    } else if (tmp->next->data == stmp->data) {
+      /* This flips the non-selected element forwards, ala bubblesort */
+      void *swap = tmp->data;
+      tmp->data = tmp->next->data;
+      tmp->next->data = swap;
+      stmp = g_list_next(stmp);
+    }
+  }
+
+  layer_set_object_list(dia->data->active_layer, new_list);
+
+  undo_reorder_objects(dia, g_list_copy(sorted_list), orig_list);
+
+  diagram_modified(dia);
+  diagram_flush(dia);
+  undo_set_transactionpoint(dia->undo);
+}
+
 void
 diagram_set_filename(Diagram *dia, char *filename)
 {
