@@ -30,21 +30,19 @@
 
 #include "intl.h"
 
-#include <tree.h>
-#include <parser.h>
-#include <xmlmemory.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 #include "dia_xml_libxml.h"
 #include "dia_xml.h"
 
 #include "load_save.h"
 #include "group.h"
+#include "diagramdata.h"
 #include "message.h"
 #include "preferences.h"
 #include "diapagelayout.h"
-
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define XML2
-#endif
+#include "autosave.h"
 
 #ifdef G_OS_WIN32
 #include <io.h>
@@ -68,7 +66,8 @@ GHFuncUnknownObjects(gpointer key,
 }
 
 static GList *
-read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
+read_objects(xmlNodePtr objects, Layer *layer,
+             GHashTable *objects_hash,const char *filename)
 {
   GList *list;
   ObjectType *type;
@@ -116,6 +115,7 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
       else
       {
         obj = type->ops->load(obj_node, version, filename);
+        layer_add_object(layer,obj);
         list = g_list_append(list, obj);
       
         g_hash_table_insert(objects_hash, g_strdup((char *)id), obj);
@@ -123,7 +123,10 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
       if (typestr) xmlFree(typestr);
       if (id) xmlFree (id);
     } else if (strcmp(obj_node->name, "group")==0) {
-      obj = group_create(read_objects(obj_node, objects_hash, filename));
+      obj = group_create(read_objects(obj_node, layer,
+                                      objects_hash, filename));
+      layer_add_object(layer,obj);
+      
       list = g_list_append(list, obj);
     } else {
       /* silently ignore other nodes */
@@ -239,7 +242,6 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 {
   GHashTable *objects_hash;
   int fd;
-  struct stat stat_buf;
   GList *list;
   xmlDocPtr doc;
   xmlNodePtr diagramdata;
@@ -249,6 +251,11 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   Layer *layer;
   xmlNsPtr namespace;
   
+  if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
+    message_error(_("You must specify a file, not a directory.\n"));
+    return FALSE;
+  }
+
   fd = open(filename, O_RDONLY);
 
   if (fd==-1) {
@@ -256,12 +263,6 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     return FALSE;
   }
 
-  fstat(fd, &stat_buf);
-  if (S_ISDIR(stat_buf.st_mode)) {
-    message_error(_("You must specify a file, not a directory.\n"));
-    return FALSE;
-  }
-  
   close(fd);
 
   doc = xmlDiaParseFile(filename);
@@ -422,7 +423,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   objects_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
   while (layer_node != NULL) {
-    utfchar *name;
+    gchar *name;
     char *visible;
     
     if (xmlIsBlankNode(layer_node)) {
@@ -437,15 +438,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 
     visible = (char *)xmlGetProp(layer_node, "visible");
 
-#ifndef UNICODE_WORK_IN_PROGRESS
-    { 
-      gchar *locname = charconv_utf8_to_local8(name);
-      layer = new_layer(locname);
-      g_free (locname);
-    }
-#else
-    layer = new_layer(g_strdup(name));
-#endif
+    layer = new_layer(g_strdup(name), data);
     if (name) xmlFree(name);
 
     layer->visible = FALSE;
@@ -455,7 +448,7 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 
     /* Read in all objects: */
     
-    list = read_objects(layer_node, objects_hash, filename);
+    list = read_objects(layer_node, layer, objects_hash, filename);
     layer->objects = list;
     read_connections( list, layer_node, objects_hash);
 
@@ -593,10 +586,10 @@ write_connections(GList *objects, xmlNodePtr layer_node,
   return TRUE;
 }
 
-static int
-diagram_data_save(DiagramData *data, const char *filename)
+/* Filename seems to be junk, but is passed on to objects */
+static xmlDocPtr
+diagram_data_write_doc(DiagramData *data, const char *filename)
 {
-  FILE *file;
   xmlDocPtr doc;
   xmlNodePtr tree;
   xmlNodePtr pageinfo, gridinfo, guideinfo;
@@ -608,38 +601,6 @@ diagram_data_save(DiagramData *data, const char *filename)
   Layer *layer;
   AttributeNode attr;
   xmlNs *name_space;
-  char *bakname,*tmpname,*dirname,*p;
-  int mode,_umask;
-  int fildes;
-  int ret;
-   
-  /* build the temporary and backup file names */
-  dirname = g_strdup(filename);
-  p = strrchr((char *)dirname,G_DIR_SEPARATOR);
-  if (p) {
-    *(p+1) = 0;
-  } else {
-    g_free(dirname);
-    dirname = g_strdup("." G_DIR_SEPARATOR_S);
-  }
-  tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
-  bakname = g_strconcat(filename,"~",NULL);
-
-  /* open a temporary name, and fix the modes to match what fopen() would have
-     done (mkstemp() is (rightly so) a bit paranoid for what we do). */
-  fildes = mkstemp(tmpname);
-  _umask = umask(0); umask(_umask);
-  mode = 0666 & ~_umask;
-  ret = fchmod(fildes,mode);
-  file = fdopen(fildes,"wb");
-
-  /* Now write the data in the temporary file name. */
-
-  if (file==NULL) {
-    message_error(_("Couldn't open: '%s' for writing.\n"), tmpname);
-    return FALSE;
-  }
-  fclose(file);
 
   doc = xmlNewDoc("1.0");
   doc->encoding = xmlStrdup("UTF-8");
@@ -687,9 +648,9 @@ diagram_data_save(DiagramData *data, const char *filename)
   data_add_real(composite_add_attribute(gridinfo, "width_y"),
 		data->grid.width_y);
   data_add_int(composite_add_attribute(gridinfo, "visible_x"),
-		data->grid.visible_x);
+	       data->grid.visible_x);
   data_add_int(composite_add_attribute(gridinfo, "visible_y"),
-		data->grid.visible_y);
+	       data->grid.visible_y);
 
   attr = new_attribute((ObjectNode)tree, "guides");
   guideinfo = data_add_composite(attr, "guides");
@@ -707,14 +668,7 @@ diagram_data_save(DiagramData *data, const char *filename)
   for (i = 0; i < data->layers->len; i++) {
     layer_node = xmlNewChild(doc->xmlRootNode, name_space, "layer", NULL);
     layer = (Layer *) g_ptr_array_index(data->layers, i);
-#ifndef UNICODE_WORK_IN_PROGRESS
-    { gchar *layername = charconv_local8_to_utf8(layer->name);
-    xmlSetProp(layer_node, "name", layername);
-    g_free(layername);
-    }
-#else
     xmlSetProp(layer_node, "name", layer->name);
-#endif
 
     if (layer->visible)
       xmlSetProp(layer_node, "visible", "true");
@@ -725,8 +679,9 @@ diagram_data_save(DiagramData *data, const char *filename)
 		  objects_hash, &obj_nr, filename);
   
     res = write_connections(layer->objects, layer_node, objects_hash);
+    /* Why do we bail out like this?  It leaks! */
     if (!res)
-      return FALSE;
+      return NULL;
   }
   g_hash_table_destroy(objects_hash);
 
@@ -735,9 +690,66 @@ diagram_data_save(DiagramData *data, const char *filename)
   else
     xmlSetDocCompressMode(doc, 0);
 
-  ret = xmlDiaSaveFile (tmpname, doc);
+  return doc;
+}
+
+/** This tries to save the diagram into a file, without any backup
+ * Returns >= 0 on success.
+ * Only for internal use. */
+static int
+diagram_data_raw_save(DiagramData *data, const char *filename)
+{
+  xmlDocPtr doc;
+  int ret;
+
+  doc = diagram_data_write_doc(data, filename);
+
+  ret = xmlDiaSaveFile (filename, doc);
   xmlFreeDoc(doc);
 
+  return ret;
+}
+
+/** This saves the diagram, using a backup in case of failure */
+static int
+diagram_data_save(DiagramData *data, const char *filename)
+{
+  FILE *file;
+  char *bakname,*tmpname,*dirname,*p;
+  int mode,_umask;
+  int fildes;
+  int ret;
+   
+  /* build the temporary and backup file names */
+  dirname = g_strdup(filename);
+  p = strrchr((char *)dirname,G_DIR_SEPARATOR);
+  if (p) {
+    *(p+1) = 0;
+  } else {
+    g_free(dirname);
+    dirname = g_strdup("." G_DIR_SEPARATOR_S);
+  }
+  tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
+  bakname = g_strconcat(filename,"~",NULL);
+
+  /* open a temporary name, and fix the modes to match what fopen() would have
+     done (mkstemp() is (rightly so) a bit paranoid for what we do). */
+  fildes = mkstemp(tmpname);
+  _umask = umask(0); umask(_umask);
+  mode = 0666 & ~_umask;
+  ret = fchmod(fildes,mode);
+  file = fdopen(fildes,"wb");
+
+  /* Now write the data in the temporary file name. */
+
+  if (file==NULL) {
+    message_error(_("Couldn't open: '%s' for writing.\n"), tmpname);
+    return FALSE;
+  }
+  fclose(file);
+
+  ret = diagram_data_raw_save(data, tmpname);
+
   if (ret < 0) {
     /* Save failed; we clean our stuff up, without touching the file named
        "filename" if it existed. */
@@ -771,9 +783,59 @@ diagram_save(Diagram *dia, const char *filename)
   dia->unsaved = FALSE;
   diagram_set_modified (dia, FALSE);
 
+  diagram_cleanup_autosave(dia);
+
   return TRUE;
 }
 
+/* Autosave stuff.  Needs to use low-level save to avoid setting and resetting flags */
+void
+diagram_cleanup_autosave(Diagram *dia)
+{
+  gchar *savefile;
+  struct stat statbuf;
+
+  savefile = dia->autosavefilename;
+
+  if (savefile == NULL) return;
+
+  if (stat(savefile, &statbuf) == 0) { /* Success */
+    unlink(savefile);
+  }
+  g_free(savefile);
+  dia->autosavefilename = NULL;
+  dia->autosaved = FALSE;
+}
+
+/** Absolutely autosave a diagram.
+ * Called after a periodic check at the first idleness.
+ */
+void
+diagram_autosave(Diagram *dia)
+{
+  gchar *save_filename;
+
+  /* Must check if the diagram is still valid, or Death Ensues! */
+  GList *diagrams = dia_open_diagrams();
+  Diagram *diagram;
+  while (diagrams != NULL) {
+    diagram = (Diagram *)diagrams->data;
+    if (diagram == dia &&
+	diagram->modified && 
+	!diagram->autosaved) {
+      save_filename = g_strdup_printf("%s.autosave", dia->filename);
+
+      if (dia->autosavefilename != NULL) 
+	g_free(dia->autosavefilename);
+      dia->autosavefilename = save_filename;
+      diagram_data_raw_save(dia->data, save_filename);
+      dia->autosaved = TRUE;
+      return;
+    }
+    diagrams = g_list_next(diagrams);
+  }
+}
+
 /* --- filter interfaces --- */
 static void
 export_native(DiagramData *data, const gchar *filename,
