@@ -58,7 +58,7 @@ static GnomeUIInfo toolbox_filemenu[] = {
                         sheets_dialog_show_callback),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_PREFERENCES_ITEM(file_preferences_callback, NULL),
-  GNOMEUIINFO_ITEM_NONE(N_("P_lugins"), NULL, file_plugins_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("P_lugins..."), NULL, file_plugins_callback),
   GNOMEUIINFO_SEPARATOR,
     /* recent file list is dynamically inserted here */
   GNOMEUIINFO_SEPARATOR,
@@ -98,7 +98,7 @@ static GnomeUIInfo editmenu[] = {
   { GNOME_APP_UI_ITEM, N_("_Delete"), NULL,
     edit_delete_callback, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, "Menu_Clear",
-    GDK_D, GDK_CONTROL_MASK },
+    GDK_DELETE, 0 },
   GNOMEUIINFO_MENU_UNDO_ITEM(edit_undo_callback, NULL),
   GNOMEUIINFO_MENU_REDO_ITEM(edit_redo_callback, NULL),
   GNOMEUIINFO_ITEM_NONE(N_("Copy Text"), NULL, edit_copy_text_callback),
@@ -112,14 +112,15 @@ static GnomeUIInfo editmenu[] = {
           NULL, GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, NULL }
 
 static GnomeUIInfo zoommenu[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA("400%",  NULL, view_zoom_set_callback, 4000),
+  GNOMEUIINFO_ITEM_NONE_DATA("_400%",  NULL, view_zoom_set_callback, 4000),
   GNOMEUIINFO_ITEM_NONE_DATA("283%",  NULL, view_zoom_set_callback, 2828),
-  GNOMEUIINFO_ITEM_NONE_DATA("200%",  NULL, view_zoom_set_callback, 2000),
+  GNOMEUIINFO_ITEM_NONE_DATA("_200%",  NULL, view_zoom_set_callback, 2000),
+/*  GNOMEUIINFO_ITEM_NONE_DATA("150%",  NULL, view_zoom_set_callback, 1500),   TODO:TestMe! */
   GNOMEUIINFO_ITEM_NONE_DATA("141%",  NULL, view_zoom_set_callback, 1414),
-  GNOMEUIINFO_ITEM_NONE_DATA("100%",  NULL, view_zoom_set_callback, 1000),
+  GNOMEUIINFO_ITEM_NONE_DATA("_100%",  NULL, view_zoom_set_callback, 1000),
   GNOMEUIINFO_ITEM_NONE_DATA("85%",   NULL, view_zoom_set_callback, 850),
   GNOMEUIINFO_ITEM_NONE_DATA("70.7%", NULL, view_zoom_set_callback, 707),
-  GNOMEUIINFO_ITEM_NONE_DATA("50%",   NULL, view_zoom_set_callback, 500),
+  GNOMEUIINFO_ITEM_NONE_DATA("_50%",   NULL, view_zoom_set_callback, 500),
   GNOMEUIINFO_ITEM_NONE_DATA("35.4%", NULL, view_zoom_set_callback, 354),
   GNOMEUIINFO_ITEM_NONE_DATA("25%",   NULL, view_zoom_set_callback, 250),
   GNOMEUIINFO_END
@@ -129,12 +130,14 @@ static GnomeUIInfo viewmenu[] = {
   GNOMEUIINFO_ITEM_NONE(N_("Zoom _In"), N_("Zoom in 50%"), view_zoom_in_callback),
   GNOMEUIINFO_ITEM_NONE(N_("Zoom _Out"), N_("Zoom out 50%"), view_zoom_out_callback),
   GNOMEUIINFO_SUBTREE(N_("_Zoom"), zoommenu),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE(N_("Diagram Properties..."), NULL, view_diagram_properties_callback),
 #ifdef HAVE_LIBART  
   GNOMEUIINFO_TOGGLEITEM(N_("_AntiAliased"), NULL,
 			 view_aa_callback, NULL),
 #endif  
-  GNOMEUIINFO_TOGGLEITEM(N_("_Visible Grid"), NULL,
+/* TODO: Show Menu Bar  F9.  Hub's cool code.  */
+  GNOMEUIINFO_TOGGLEITEM(N_("Show _Grid"), NULL,
 			 view_visible_grid_callback, NULL),
   GNOMEUIINFO_TOGGLEITEM(N_("_Snap To Grid"), NULL,
 			 view_snap_to_grid_callback, NULL),
@@ -145,6 +148,7 @@ static GnomeUIInfo viewmenu[] = {
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE(N_("New _View"), NULL, view_new_view_callback),
   GNOMEUIINFO_ITEM_NONE(N_("Show _All"), NULL, view_show_all_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("Redraw"), NULL, view_redraw_callback),
   GNOMEUIINFO_END
 };
 
@@ -163,8 +167,8 @@ static GnomeUIInfo selecttype_radiolist[] = {
 };
 
 static GnomeUIInfo selectmenu[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("All"), NULL, select_all_callback, 0),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("None"), NULL, select_none_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("All"), "<ctrl>A", select_all_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("None"), "<ctrl><shift>A", select_none_callback, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Invert"), NULL, select_invert_callback, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Connected"), NULL, select_connected_callback, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Transitive"), NULL, select_transitive_callback, 0),
@@ -178,6 +182,7 @@ static GnomeUIInfo objects_align_h[] = {
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Left"), NULL, objects_align_h_callback, DIA_ALIGN_LEFT),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_h_callback, DIA_ALIGN_CENTER),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Right"), NULL, objects_align_h_callback, DIA_ALIGN_RIGHT),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_h_callback, DIA_ALIGN_EQUAL),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,DIA_ALIGN_ADJACENT),
   GNOMEUIINFO_END
@@ -185,8 +190,9 @@ static GnomeUIInfo objects_align_h[] = {
 
 static GnomeUIInfo objects_align_v[] = {
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Top"), NULL, objects_align_v_callback, DIA_ALIGN_TOP),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_v_callback, DIA_ALIGN_CENTER),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Middle"), NULL, objects_align_v_callback, DIA_ALIGN_CENTER),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Bottom"), NULL, objects_align_v_callback, DIA_ALIGN_BOTTOM),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_v_callback, DIA_ALIGN_EQUAL),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback, DIA_ALIGN_ADJACENT),
   GNOMEUIINFO_END
@@ -195,6 +201,8 @@ static GnomeUIInfo objects_align_v[] = {
 static GnomeUIInfo objectsmenu[] = {
   GNOMEUIINFO_ITEM_NONE(N_("Send to _Back"), NULL, objects_place_under_callback),
   GNOMEUIINFO_ITEM_NONE(N_("Bring to _Front"), NULL, objects_place_over_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("Send Backwards"), NULL, objects_place_down_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("Bring Forwards"), NULL, objects_place_up_callback),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE(N_("_Group"), NULL, objects_group_callback),
   GNOMEUIINFO_ITEM_NONE(N_("_Ungroup"), NULL, objects_ungroup_callback),
@@ -213,11 +221,13 @@ static GnomeUIInfo toolsmenu[] = {
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Ellipse"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Polygon"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Beziergon"), NULL, NULL, 0),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Line"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Arc"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Zigzagline"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Polyline"), NULL, NULL, 0),
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Bezierline"), NULL, NULL, 0),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_ITEM_NONE_DATA(N_("Image"), NULL, NULL, 0),
   GNOMEUIINFO_END
 };
@@ -229,7 +239,7 @@ static GnomeUIInfo dialogsmenu[] = {
 };
 
 static GnomeUIInfo helpmenu[] = {
-  GNOMEUIINFO_HELP("dia"),
+  GNOMEUIINFO_HELP("Dia"),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_ABOUT_ITEM(help_about_callback, NULL),
   GNOMEUIINFO_END
@@ -241,6 +251,11 @@ static GnomeUIInfo toolbox_menu[] = {
   GNOMEUIINFO_END
 };
 
+static GnomeUIInfo inputmethod_menu[] = {
+  GNOMEUIINFO_END
+};
+
+
 static GnomeUIInfo display_menu[] = {
   GNOMEUIINFO_MENU_FILE_TREE(filemenu),
   GNOMEUIINFO_MENU_EDIT_TREE(editmenu),
@@ -249,6 +264,7 @@ static GnomeUIInfo display_menu[] = {
   GNOMEUIINFO_SUBTREE(N_("_Objects"), objectsmenu),
   GNOMEUIINFO_SUBTREE(N_("_Tools"), toolsmenu),
   GNOMEUIINFO_SUBTREE(N_("_Dialogs"), dialogsmenu),
+  GNOMEUIINFO_SUBTREE(N_("_Input Methods"), inputmethod_menu),
   GNOMEUIINFO_END
 };
 
@@ -261,25 +277,30 @@ static GtkItemFactoryEntry toolbox_menu_items[] =
 {
   {N_("/_File"),               NULL,         NULL,       0,    "<Branch>" },
   {   "/File/tearoff",         NULL,         NULL,       0,   "<Tearoff>" },
-  {N_("/File/_New"),           "<control>N", file_new_callback,         0 },
-  {N_("/File/_Open"),          "<control>O", file_open_callback,        0 },
+  {N_("/File/_New"),           "<control>N", file_new_callback,         0,
+      "<StockItem>", GTK_STOCK_NEW },
+  {N_("/File/_Open..."),       "<control>O", file_open_callback,        0,
+      "<StockItem>", GTK_STOCK_OPEN },
   {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/File/_Diagram tree"),  NULL,         diagtree_show_callback,    0,
    "<ToggleItem>" },
   {N_("/File/Sheets and Objects..."),
                                NULL,         sheets_dialog_show_callback, 0 },
   {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
-  {N_("/File/_Preferences..."),NULL,         file_preferences_callback, 0 },
-  {N_("/File/P_lugins"),       NULL,         file_plugins_callback,     0 },
+ {N_("/File/_Preferences..."),NULL,         file_preferences_callback, 0,
+      "<StockItem>", GTK_STOCK_PREFERENCES },
+   {N_("/File/P_lugins..."),   NULL,         file_plugins_callback,     0 },
   {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
     /* recent file list is dynamically inserted here */
   {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
-  {N_("/File/_Quit"),          "<control>Q", file_quit_callback,        0 },
+  {N_("/File/_Quit"),          "<control>Q", file_quit_callback,        0,
+      "<StockItem>", GTK_STOCK_QUIT },
   {N_("/_Help"),               NULL,         NULL,       0,    "<Branch>" },
   {   "/Help/tearoff",         NULL,         NULL,       0,   "<Tearoff>" },
-  {N_("/Help/_Manual"),        "F1",         help_manual_callback,      0 },
+  {N_("/Help/_Manual"),        "F1",         help_manual_callback,      0,
+      "<StockItem>", GTK_STOCK_HELP },
   {N_("/Help/---"),            NULL,         NULL,       0, "<Separator>" },
-  {N_("/Help/_About"),         NULL,         help_about_callback,       0 },
+  {N_("/Help/_About..."),      NULL,         help_about_callback,       0 },
 };
 
 /* calculate the number of menu_item's */
@@ -291,60 +312,81 @@ static GtkItemFactoryEntry display_menu_items[] =
   {   "/tearoff",                 NULL,         NULL,         0, "<Tearoff>" },
   {N_("/_File"),                  NULL,         NULL,           0, "<Branch>"},
   {   "/File/tearoff",            NULL,         NULL,         0, "<Tearoff>" },
-  {N_("/File/_New"),              "<control>N", file_new_callback,          0},
-  {N_("/File/_Open"),             "<control>O", file_open_callback,         0},
-  {N_("/File/_Save"),             "<control>S", file_save_callback,         0},
-  {N_("/File/Save _As..."),       "<control>W", file_save_as_callback,      0},
+  {N_("/File/_New"),              "<control>N", file_new_callback,          0,
+      "<StockItem>", GTK_STOCK_NEW },
+  {N_("/File/_Open..."),          "<control>O", file_open_callback,         0,
+      "<StockItem>", GTK_STOCK_OPEN },
+  {N_("/File/_Save"),             "<control>S", file_save_callback,         0,
+      "<StockItem>", GTK_STOCK_SAVE },
+  {N_("/File/Save _As..."),       "<control><shift>S", file_save_as_callback,      0,
+      "<StockItem>", GTK_STOCK_SAVE_AS },
   {N_("/File/_Export..."),        NULL,         file_export_callback,       0},
   {N_("/File/---"),               NULL,         NULL,        0, "<Separator>"},
   {N_("/File/Page Set_up..."),    NULL,         file_pagesetup_callback,    0},
-  {N_("/File/_Print Diagram..."), "<control>P", file_print_callback,        0},
+  {N_("/File/_Print Diagram..."), "<control>P", file_print_callback,        0,
+      "<StockItem>", GTK_STOCK_PRINT },
   {N_("/File/---"),               NULL,         NULL,        0, "<Separator>"},
-  {N_("/File/_Close"),            NULL,         file_close_callback,        0},
+  {N_("/File/_Close"),            "<control>W",         file_close_callback,        0,
+      "<StockItem>", GTK_STOCK_CLOSE },
   {   "/File/---MRU",             NULL,         NULL,        0, "<Separator>"},
-  {N_("/File/_Quit"),              "<control>Q", file_quit_callback,        0},
+  {N_("/File/_Quit"),              "<control>Q", file_quit_callback,        0,
+      "<StockItem>", GTK_STOCK_QUIT},
   {N_("/_Edit"),                  NULL,         NULL,           0, "<Branch>"},
   {   "/Edit/tearoff",            NULL,         NULL,         0, "<Tearoff>" },
-  {N_("/Edit/_Copy"),             "<control>C", edit_copy_callback,         0},
-  {N_("/Edit/C_ut"),              "<control>X", edit_cut_callback,          0},
-  {N_("/Edit/_Paste"),            "<control>V", edit_paste_callback,        0},
-  {N_("/Edit/_Delete"),           "<control>D", edit_delete_callback,       0},
-  {N_("/Edit/_Undo"),             "<control>Z", edit_undo_callback,         0},
-  {N_("/Edit/_Redo"),             "<control>R", edit_redo_callback,         0},
+  {N_("/Edit/_Undo"),             "<control>Z", edit_undo_callback,         0,
+      "<StockItem>", GTK_STOCK_UNDO },
+  {N_("/Edit/_Redo"),             "<control><shift>Z", edit_redo_callback,         0,
+      "<StockItem>", GTK_STOCK_REDO },
+  {N_("/Edit/---"),            NULL,         NULL,       0, "<Separator>" },
+  {N_("/Edit/_Copy"),             "<control>C", edit_copy_callback,         0,
+      "<StockItem>", GTK_STOCK_COPY },
+  {N_("/Edit/C_ut"),              "<control>X", edit_cut_callback,          0,
+      "<StockItem>", GTK_STOCK_CUT },
+  {N_("/Edit/_Paste"),            "<control>V", edit_paste_callback,        0,
+      "<StockItem>", GTK_STOCK_PASTE },
+  {N_("/Edit/_Delete"),           "Delete", edit_delete_callback,       0,
+      "<StockItem>", GTK_STOCK_DELETE },
+  {N_("/Edit/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/Edit/Copy Text"),         NULL,         edit_copy_text_callback,    0},
   {N_("/Edit/Cut Text"),          NULL,         edit_cut_text_callback,     0},
   {N_("/Edit/Paste _Text"),       NULL,         edit_paste_text_callback,   0},
   {N_("/_View"),                  NULL,         NULL,           0, "<Branch>"},
   {   "/View/tearoff",            NULL,         NULL,         0, "<Tearoff>" },
-  {N_("/View/Zoom _In"),          NULL,          view_zoom_in_callback,     0},
-  {N_("/View/Zoom _Out"),         NULL,          view_zoom_out_callback,    0},
+  {N_("/View/Zoom _In"),          NULL,          view_zoom_in_callback,     0,
+      "<StockItem>", GTK_STOCK_ZOOM_IN },
+  {N_("/View/Zoom _Out"),         NULL,          view_zoom_out_callback,    0,
+      "<StockItem>", GTK_STOCK_ZOOM_OUT },
   {N_("/View/_Zoom"),             NULL,         NULL,           0, "<Branch>"},
   {   "/View/Zoom/tearoff",       NULL,         NULL,         0, "<Tearoff>" },
   {N_("/View/Zoom/400%"),         NULL,         view_zoom_set_callback,  4000},
   {N_("/View/Zoom/283%"),         NULL,         view_zoom_set_callback,  2828},
   {N_("/View/Zoom/200%"),         NULL,         view_zoom_set_callback,  2000},
   {N_("/View/Zoom/141%"),         NULL,         view_zoom_set_callback,  1414},
-  {N_("/View/Zoom/100%"),         "1",          view_zoom_set_callback,  1000},
+  {N_("/View/Zoom/100%"),         NULL,         view_zoom_set_callback,  1000,
+      "<StockItem>", GTK_STOCK_ZOOM_100 },
   {N_("/View/Zoom/85%"),          NULL,         view_zoom_set_callback,   850},
   {N_("/View/Zoom/70.7%"),        NULL,         view_zoom_set_callback,   707},
   {N_("/View/Zoom/50%"),          NULL,         view_zoom_set_callback,   500},
   {N_("/View/Zoom/35.4%"),        NULL,         view_zoom_set_callback,   354},
   {N_("/View/Zoom/25%"),          NULL,         view_zoom_set_callback,   250},
+  {N_("/View/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/View/Diagram Properties..."),NULL,         view_diagram_properties_callback, 0},
 #ifdef HAVE_LIBART  
   {N_("/View/_AntiAliased"),      NULL,         view_aa_callback,           0, "<ToggleItem>"},
 #endif
-  {N_("/View/_Visible Grid"),     NULL,         view_visible_grid_callback, 0, "<ToggleItem>"},
-  {N_("/View/_Snap To Grid"),     "G",          view_snap_to_grid_callback, 0, "<ToggleItem>"},
+  {N_("/View/Show _Grid"),     NULL,         view_visible_grid_callback, 0, "<ToggleItem>"},
+  {N_("/View/_Snap To Grid"),     NULL,         view_snap_to_grid_callback, 0, "<ToggleItem>"},
   {N_("/View/Show _Rulers"),      NULL,         view_toggle_rulers_callback,0, "<ToggleItem>"},
   {N_("/View/Show _Connection Points"),	  NULL,		view_show_cx_pts_callback,   0,	"<ToggleItem>"},
   {N_("/View/---"),               NULL,         NULL,        0, "<Separator>"},
-  {N_("/View/New _View"),         "<control>I", view_new_view_callback,     0},
-  {N_("/View/Show _All"),         "<control>A", view_show_all_callback,     0},
+  {N_("/View/New _View"),         NULL, view_new_view_callback,     0},
+  /* Show All, Best Fit.  Same as the Gimp, Ctrl+E */
+  {N_("/View/Show _All"),         "<control>E", view_show_all_callback,     0},
+  {N_("/View/Re_draw"),           NULL,         view_redraw_callback,       0},
   {N_("/_Select"),                NULL,         NULL,           0, "<Branch>"},
   {   "/Select/tearoff",          NULL,         NULL,         0, "<Tearoff>" },
-  {N_("/Select/All"),             NULL,         select_all_callback,        0},
-  {N_("/Select/None"),            NULL,         select_none_callback,       0},
+  {N_("/Select/All"),             "<control>A", select_all_callback,        0},
+  {N_("/Select/None"),            "<ctrl><shift>A", select_none_callback,   0},
   {N_("/Select/Invert"),          NULL,         select_invert_callback,     0},
   {N_("/Select/Connected"),       NULL,         select_connected_callback,  0},
   {N_("/Select/Transitive"),      NULL,         select_transitive_callback, 0},
@@ -364,22 +406,27 @@ static GtkItemFactoryEntry display_menu_items[] =
   {   "/Objects/tearoff",         NULL,         NULL,         0, "<Tearoff>" },
   {N_("/Objects/Send to _Back"),  "<control>B",objects_place_under_callback,0},
   {N_("/Objects/Bring to _Front"),"<control>F", objects_place_over_callback,0},
+  {N_("/Objects/Send Backwards"),  "<control><shift>B",objects_place_down_callback,0},
+  {N_("/Objects/Bring Forwards"),"<control><shift>F", objects_place_up_callback,0},
   {N_("/Objects/---"),            NULL,         NULL,        0, "<Separator>"},
   {N_("/Objects/_Group"),         "<control>G", objects_group_callback,     0},
-  {N_("/Objects/_Ungroup"),       "<control>U", objects_ungroup_callback,   0},
+  /* deliberately not using Ctrl+U for Ungroup */
+  {N_("/Objects/_Ungroup"),       "<control><shift>G", objects_ungroup_callback,   0}, 
   {N_("/Objects/---"),            NULL,         NULL,        0, "<Separator>"},
   {N_("/Objects/Align _Horizontal"),       NULL, NULL,          0, "<Branch>"},
   {   "/Objects/Align Horizontal/tearoff", NULL, NULL,        0, "<Tearoff>" },
   {N_("/Objects/Align Horizontal/Left"),   NULL, objects_align_h_callback,  DIA_ALIGN_LEFT},
   {N_("/Objects/Align Horizontal/Center"), NULL, objects_align_h_callback,  DIA_ALIGN_CENTER},
   {N_("/Objects/Align Horizontal/Right"),  NULL, objects_align_h_callback,  DIA_ALIGN_RIGHT},
+  {N_("/Objects/Align Horizontal/---"),             NULL,         NULL,        0, "<Separator>"},
   {N_("/Objects/Align Horizontal/Equal Distance"), NULL, objects_align_h_callback,    DIA_ALIGN_EQUAL},
   {N_("/Objects/Align Horizontal/Adjacent"), NULL, objects_align_h_callback,    DIA_ALIGN_ADJACENT},
   {N_("/Objects/Align _Vertical"),         NULL, NULL,          0, "<Branch>"},
   {   "/Objects/Align Vertical/tearoff",   NULL, NULL,        0, "<Tearoff>" },
   {N_("/Objects/Align Vertical/Top"),      NULL, objects_align_v_callback,  DIA_ALIGN_TOP},
-  {N_("/Objects/Align Vertical/Center"),   NULL, objects_align_v_callback,  DIA_ALIGN_CENTER},
+  {N_("/Objects/Align Vertical/Middle"),   NULL, objects_align_v_callback,  DIA_ALIGN_CENTER},
   {N_("/Objects/Align Vertical/Bottom"),   NULL, objects_align_v_callback,  DIA_ALIGN_BOTTOM},
+  {N_("/Objects/Align Vertical/---"),             NULL,         NULL,        0, "<Separator>"}, 
   {N_("/Objects/Align Vertical/Equal Distance"),   NULL, objects_align_v_callback,    DIA_ALIGN_EQUAL},
   {N_("/Objects/Align Vertical/Adjacent"), NULL, objects_align_v_callback,  DIA_ALIGN_ADJACENT},
 
@@ -393,17 +440,21 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/Tools/Ellipse"),          NULL,     NULL,                           0},
   {N_("/Tools/Polygon"),          NULL,     NULL,                           0},
   {N_("/Tools/Beziergon"),        NULL,     NULL,                           0},
+  {N_("/Tools/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/Tools/Line"),             NULL,     NULL,                           0},
   {N_("/Tools/Arc"),              NULL,     NULL,                           0},
   {N_("/Tools/Zigzagline"),       NULL,     NULL,                           0},
   {N_("/Tools/Polyline"),         NULL,     NULL,                           0},
   {N_("/Tools/Bezierline"),       NULL,     NULL,                           0},
+  {N_("/Tools/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/Tools/Image"),            NULL,     NULL,                           0},
-
   {N_("/_Dialogs"),               NULL,     NULL,               0, "<Branch>"},
   {   "/Dialogs/tearoff",         NULL,         NULL,         0, "<Tearoff>" },
-  {N_("/Dialogs/_Properties"),    NULL,     dialogs_properties_callback,    0},
-  {N_("/Dialogs/_Layers"),        NULL,     dialogs_layers_callback,        0},
+  {N_("/Dialogs/_Properties..."), NULL,     dialogs_properties_callback,    0},
+  {N_("/Dialogs/_Layers..."),     NULL,     dialogs_layers_callback,        0},
+
+  {N_("/_Input Methods"),         NULL,     NULL,               0, "<Branch>"},
+  {   "/Input Methods/tearoff",   NULL,     NULL,               0, "<Tearoff>" },
 };
 
 static int display_nmenu_items = (sizeof(display_menu_items) /
@@ -589,7 +640,7 @@ save_accels(gpointer data)
 
   accelfilename = dia_config_filename("menurc");
   if (accelfilename) {
-    gtk_item_factory_dump_rc (accelfilename, NULL, TRUE);
+    gtk_accel_map_save (accelfilename);
     g_free (accelfilename);
   }
   return TRUE;
@@ -620,7 +671,7 @@ dia_menu_signal_connect_func (GnomeUIInfo *uiinfo, gchar *signal_name,
 {
     /* only connect the signal if uiinfo->moreinfo not NULL */
     /* otherwise it is non sens and generate extraneous warnings */
-    /* see bug 55047 <http:/*bugzilla.gnome.org/show_bug.cgi?id=55047> */
+    /* see bug 55047 <http://bugzilla.gnome.org/show_bug.cgi?id=55047> */
     if (uiinfo->moreinfo != NULL) {
         gtk_signal_connect (GTK_OBJECT(uiinfo->widget), signal_name,
                             GTK_SIGNAL_FUNC(dia_menu_signal_proxy), uiinfo);
@@ -628,7 +679,7 @@ dia_menu_signal_connect_func (GnomeUIInfo *uiinfo, gchar *signal_name,
 }
 
 static GnomeUIBuilderData dia_menu_uibdata = {
-  dia_menu_signal_connect_func, /* connect_func */
+  (GnomeUISignalConnectFunc)dia_menu_signal_connect_func, /* connect_func */
   NULL,                         /* data */
   FALSE,                        /* is_interp */
   (GtkCallbackMarshal) 0,       /* relay_func */
@@ -644,7 +695,7 @@ menus_set_tools_callback (const char * menu_name, GtkItemFactory *item_factory)
 {
     gint i, len;
     GString *path;
-    GtkWidget *menuitem;
+    GtkMenuItem *menuitem;
     
     path = g_string_new(menu_name);
     g_string_append(path, "/Tools/");
@@ -737,20 +788,13 @@ menus_init(void)
 
   display_menus = gtk_item_factory_get_widget(display_item_factory,
 					      "<Display>");
+  gtk_menu_set_accel_path(GTK_MENU(display_menus), "<Display>/");
 #endif
 
   menus_set_tools_callback ("<Display>", NULL);
 
   gtk_menu_set_title(GTK_MENU(display_menus), _("Diagram Menu"));
   
-  accelfilename = dia_config_filename("menurc");
-  
-  if (accelfilename) {
-    gtk_item_factory_parse_rc(accelfilename);
-    g_free(accelfilename);
-  }
-  gtk_quit_add(1, save_accels, NULL);
-
   /* initialize callbacks from plug-ins */
   for (cblist = filter_get_callbacks(); cblist; cblist = cblist->next) {
     DiaCallbackFilter *cbf = cblist->data;
@@ -764,6 +808,16 @@ menus_init(void)
     gtk_signal_connect(GTK_OBJECT(newitem), "activate",
 		       GTK_SIGNAL_FUNC(plugin_callback), cbf);
   } /* for filter_callbacks */
+
+  /* load accelerators and prepare to later save them */
+  accelfilename = dia_config_filename("menurc");
+  
+  if (accelfilename) {
+    gtk_accel_map_load(accelfilename);
+    g_free(accelfilename);
+  }
+  gtk_quit_add(1, save_accels, NULL);
+
 }
 
 void
@@ -837,10 +891,10 @@ menus_get_image_menubar (GtkWidget **menu, GtkItemFactory **display_mbar_item_fa
 
  * but doesn't perform any translation of the menu items, and removes
  * underscores when performing the string comparisons */
-static GtkWidget *
+static GtkMenuItem *
 dia_gnome_menu_get_widget (GnomeUIInfo *uiinfo, const gchar *path)
 {
-  GtkWidget *ret;
+  GtkMenuItem *ret;
   gchar *name_end;
   gint path_len, label_len, i, j;
   gboolean label_matches;
@@ -893,7 +947,7 @@ dia_gnome_menu_get_widget (GnomeUIInfo *uiinfo, const gchar *path)
 	}
       }
       if (label_matches && j == path_len && i == label_len)
-	return uiinfo->widget;
+	return GTK_MENU_ITEM(uiinfo->widget);
       break;
 
     case GNOME_APP_UI_SUBTREE:
@@ -941,7 +995,8 @@ find_parent(GtkMenuShell *top, GnomeUIInfo *uiinfo, GHashTable *extras,
 {
   const gchar *slash;
   gchar *parent_path;
-  GtkWidget *parent_item, *parent_submenu, *tearoff, *grandparent_submenu;
+  GtkMenuItem *parent_item;
+  GtkWidget *parent_submenu, *tearoff, *grandparent_submenu;
 
   slash = strrchr(mpath, '/');
   if (!slash)
@@ -954,13 +1009,13 @@ find_parent(GtkMenuShell *top, GnomeUIInfo *uiinfo, GHashTable *extras,
     parent_item = dia_gnome_menu_get_widget(uiinfo, parent_path);
   if (parent_item) {
     g_free(parent_path);
-    if (GTK_MENU_ITEM(parent_item)->submenu) {
-      return GTK_MENU_SHELL(GTK_MENU_ITEM(parent_item)->submenu);
+    if (parent_item->submenu) {
+      return GTK_MENU_SHELL(parent_item->submenu);
     } else {
       GtkWidget *menu = gtk_menu_new();
 
       /* setup submenu, and add a tearoff at the top */
-      gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent_item), menu);
+      gtk_menu_item_set_submenu(parent_item, menu);
       tearoff = gtk_tearoff_menu_item_new();
       gtk_container_add(GTK_CONTAINER(menu), tearoff);
       gtk_widget_show(tearoff);
@@ -976,13 +1031,13 @@ find_parent(GtkMenuShell *top, GnomeUIInfo *uiinfo, GHashTable *extras,
     slash++;
   else
     slash = parent_path;
-  parent_item = gtk_menu_item_new_with_label(slash);
-  gtk_container_add(GTK_CONTAINER(grandparent_submenu), parent_item);
-  gtk_widget_show(parent_item);
+  parent_item = GTK_MENU_ITEM(gtk_menu_item_new_with_label(slash));
+  gtk_container_add(GTK_CONTAINER(grandparent_submenu), GTK_WIDGET(parent_item));
+  gtk_widget_show(GTK_WIDGET(parent_item));
   g_hash_table_insert(extras, parent_path, parent_item); /* add to extras */
 
   parent_submenu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent_item), parent_submenu);
+  gtk_menu_item_set_submenu(parent_item, parent_submenu);
   tearoff = gtk_tearoff_menu_item_new();
   gtk_container_add(GTK_CONTAINER(parent_submenu), tearoff);
   gtk_widget_show(tearoff);
@@ -1069,14 +1124,15 @@ menus_add_path (const gchar *path)
   In case of an item in the <DisplayMBar> menu bar, provide the corresponding 
   item factory: ddisp->mbar_item_factory.
  */
-GtkWidget *
+GtkMenuItem *
 menus_get_item_from_path (char *path, GtkItemFactory *item_factory)
 {
-  GtkWidget *widget = NULL;
+  GtkWidget *wid = NULL;
+  GtkMenuItem *widget = NULL;
 
 # ifdef GNOME
+      /* DDisplay *ddisp; */
   const gchar *menu_name;
-  DDisplay *ddisp;
 
   /* drop the "<Display>/", "<DisplayMBar>/" or "<Toolbox>/" at the start */
   menu_name = path;
@@ -1084,11 +1140,8 @@ menus_get_item_from_path (char *path, GtkItemFactory *item_factory)
   path ++; /* move past the / */
 
   if (strncmp(menu_name, "<Display>", strlen("<Display>")) == 0) {
-/*    ddisp = ddisplay_active ();
-      if (! ddisp)
-      return NULL; */
     widget = dia_gnome_menu_get_widget(display_menu, path);
-  } else if  (strncmp(menu_name, "<DisplayMBar>", strlen("<DisplayMBar>"))  == 0) {
+  } else if (strncmp(menu_name,"<DisplayMBar>",strlen("<DisplayMBar>")) == 0) {
       /* finding this requires an item factory*/
       widget = dia_gnome_menu_get_widget(display_menu, path);
   } else if (strncmp(menu_name, "<Toolbox>", strlen("<Toolbox>")) == 0) {
@@ -1097,21 +1150,27 @@ menus_get_item_from_path (char *path, GtkItemFactory *item_factory)
 # else
 
   if (display_item_factory) {
-    widget = gtk_item_factory_get_widget(display_item_factory, path);
+    wid =
+      gtk_item_factory_get_item(display_item_factory, path);
+    if (wid != NULL) widget = GTK_MENU_ITEM(wid);
   }
   
   if ((widget == NULL) && (item_factory)) {
-      widget = gtk_item_factory_get_widget(item_factory, path);
+      wid = 
+	gtk_item_factory_get_item(item_factory, path);
+    if (wid != NULL) widget = GTK_MENU_ITEM(wid);
   }
   
   if (widget == NULL) {
-    widget = gtk_item_factory_get_widget(toolbox_item_factory, path);
+    wid = 
+      gtk_item_factory_get_item(toolbox_item_factory, path);
+    if (wid != NULL) widget = GTK_MENU_ITEM(wid);
   }
 # endif
 
   if (! widget) {
-    g_warning("Can't find menu entry '%s'!\nThis is probably a i18n problem "
-	       "(try LANG=C).", path);
+    g_warning(_("Can't find menu entry '%s'!\nThis is probably a i18n problem "
+                "(try LANG=C)."), path);
   }
 
   return widget;
@@ -1172,7 +1231,7 @@ menus_initialize_updatable_items (UpdatableMenuItems *items,
     items->align_h_a = menus_get_item_from_path(path->str, factory);
     g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Top");
     items->align_v_t = menus_get_item_from_path(path->str, factory);
-    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Center");
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Middle");
     items->align_v_c = menus_get_item_from_path(path->str, factory);
     g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Bottom");
     items->align_v_b = menus_get_item_from_path(path->str, factory);
