@@ -26,9 +26,9 @@
 #include "connectionpoint_ops.h"
 #include "message.h"
 #include "properties.h"
-#include "render_gdk.h"
 #include "select.h"
 #include "preferences.h"
+#include "cursor.h"
 
 static Object *click_select_object(DDisplay *ddisp, Point *clickedpoint,
 				   GdkEventButton *event);
@@ -131,7 +131,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       
       diagram_select(diagram, obj);
       obj->ops->selectf(obj, clickedpoint,
-		       (Renderer *)ddisp->renderer);
+		        ddisp->renderer);
 
       /*
 	This stuff is buggy, fix it later.
@@ -148,7 +148,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
     } else { /* Clicked on already selected. */
       /*printf("Already selected\n");*/
       obj->ops->selectf(obj, clickedpoint,
-		       (Renderer *)ddisp->renderer);
+		       ddisp->renderer);
       object_add_updates_list(diagram->data->selected, diagram);
       diagram_flush(diagram);
       
@@ -166,6 +166,15 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
   return NULL;
 }
 
+static glong
+time_micro()
+{
+  GTimeVal tv;
+
+  g_get_current_time(&tv);
+  return tv.tv_sec*G_USEC_PER_SEC+tv.tv_usec;
+}
+
 static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
 				Point *clickedpoint, GdkEventButton *event)
 {
@@ -186,6 +195,8 @@ static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
                       GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                       NULL, NULL, event->time);
     tool->start_at = *clickedpoint;
+    tool->start_time = time_micro();
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
     return TRUE;
   }
   return FALSE;
@@ -220,6 +231,8 @@ modify_button_press(ModifyTool *tool, GdkEventButton *event,
                       GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                       NULL, NULL, event->time);
     tool->start_at = clickedpoint;
+    tool->start_time = time_micro();
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
   } else {
     tool->state = STATE_BOX_SELECT;
     tool->start_box = clickedpoint;
@@ -272,6 +285,58 @@ modify_double_click(ModifyTool *tool, GdkEventButton *event,
   }
 }
 
+#define MIN_PIXELS 10
+
+/** Makes sure that objects aren't accidentally moved when double-clicking
+ * for properties.  Objects do not move unless double click time has passed
+ * or the move is 'significant'.  Allowing the 'significant' move makes a
+ * regular grab-and-move less jerky.
+ *
+ * There's a slight chance that the user could move outside the
+ * minimum movement limit and back in within the double click time
+ * (normally .25 seconds), but that's very, very unlikely.  If
+ * it happens, the cursor wouldn't reset just right.
+ */
+
+static gboolean
+modify_move_already(ModifyTool *tool, DDisplay *ddisp, Point *to)
+{
+  static gboolean settings_taken = FALSE;
+  static int double_click_time = 250;
+  real dist;
+
+  if (!settings_taken) {
+    /* One could argue that if the settings were updated while running,
+       we should re-read them.  But I don't see any way to get notified,
+       and I don't want to do this whole thing for each bit of the
+       move --Lars */
+    GtkSettings *settings = gtk_settings_get_default();
+    if (settings == NULL) {
+      g_message(_("Couldn't get GTK settings"));
+    } else {
+      g_object_get(G_OBJECT(settings), 
+		   "gtk-double-click-time", &double_click_time, NULL);
+    }
+    settings_taken = TRUE;
+  }
+  if (tool->start_time < time_micro()-double_click_time*1000) {
+    return TRUE;
+  }
+  dist = distance_point_point_manhattan(&tool->start_at, to);
+  if (ddisp->grid.snap) {
+    real grid_x = ddisp->diagram->data->grid.width_x;
+    real grid_y = ddisp->diagram->data->grid.width_y;
+    if (dist > grid_x || dist > grid_y) {
+      return TRUE;
+    }
+  }
+  if (ddisplay_transform_length(ddisp, dist) > MIN_PIXELS) {
+    return (ddisplay_transform_length(ddisp, dist) > MIN_PIXELS);
+  } else {
+    return FALSE;
+  }
+}
+
 
 static void
 modify_motion(ModifyTool *tool, GdkEventMotion *event,
@@ -289,6 +354,8 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   
   ddisplay_untransform_coords(ddisp, event->x, event->y, &to.x, &to.y);
 
+  if (!modify_move_already(tool, ddisp, &to)) return;
+
   switch (tool->state) {
   case STATE_MOVE_OBJECT:
 
@@ -343,7 +410,7 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   case STATE_MOVE_HANDLE:
     /* Move to ConnectionPoint if near: */
     connectionpoint =
-      object_find_connectpoint_display(ddisp, &to);
+      object_find_connectpoint_display(ddisp, &to, tool->object);
 
     if (event->state & GDK_CONTROL_MASK) {
       full_delta = to;
@@ -354,9 +421,11 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
     if ( (tool->handle->connect_type != HANDLE_NONCONNECTABLE) &&
 	 (connectionpoint != NULL) ) {
       to = connectionpoint->pos;
+      ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
     } else {
       /* No connectionopoint near, then snap to grid (if enabled) */
       snap_to_grid(ddisp, &to.x, &to.y);
+      ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
     }
 
     if (tool->break_connections) {
@@ -431,14 +500,26 @@ static void
 modify_button_release(ModifyTool *tool, GdkEventButton *event,
 		      DDisplay *ddisp)
 {
-  Point *dest_pos;
+  Point *dest_pos, to;
   GList *list;
   int i;
   Object *obj;
   
+  tool->break_connections = FALSE;
+  ddisplay_set_all_cursor(default_cursor);
+
   switch (tool->state) {
   case STATE_MOVE_OBJECT:
+    /* Return to normal state */
     gdk_pointer_ungrab (event->time);
+
+    ddisplay_untransform_coords(ddisp, event->x, event->y, &to.x, &to.y);
+    if (!modify_move_already(tool, ddisp, &to)) {
+      tool->orig_pos = NULL;
+      tool->state = STATE_NONE;
+      return;
+    }
+
     diagram_update_connections_selection(ddisp->diagram);
 
     if (tool->orig_pos != NULL) {
@@ -467,6 +548,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     break;
   case STATE_MOVE_HANDLE:
     gdk_pointer_ungrab (event->time);
+    tool->state = STATE_NONE;
 
     if (tool->orig_pos != NULL) {
       undo_move_handle(ddisp->diagram, tool->handle, tool->object,
@@ -498,9 +580,6 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       tool->orig_pos = NULL;
     }
 
-
-    
-    tool->state = STATE_NONE;
     break;
   case STATE_BOX_SELECT:
     gdk_pointer_ungrab (event->time);
@@ -598,7 +677,6 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     message_error("Internal error: Strange state in modify_tool\n");
       
   }
-  tool->break_connections = FALSE;
 }
 
 
