@@ -18,7 +18,10 @@
 #ifndef ARROWS_H
 #define ARROWS_H
 
+#include "diavar.h"
+#include "diatypes.h"
 #include "geometry.h"
+#include "color.h"
 
 /* NOTE: Add new arrow types at the end, or the enums
    will change order leading to file incompatibilities. */
@@ -47,25 +50,39 @@ typedef enum {
   ARROW_SLASH_ARROW,             /* {slash arrow} Vertical + diagonal line*/
   ARROW_INTEGRAL_SYMBOL,         /* {integral symbol} Vertical + integral */
   ARROW_CROW_FOOT,
-  ARROW_CROSS                    /* Vertical line */
+  ARROW_CROSS,                   /* Vertical line */
+  ARROW_FILLED_CONCAVE,
+  ARROW_BLANKED_CONCAVE,
 } ArrowType;
 
-typedef struct {
+struct menudesc {
+  char *name;
+  int enum_value;
+};
+
+/* These are used to fill menus.  See dia_arrow_fill_menu in widgets.c */
+DIAVAR struct menudesc arrow_types[];
+
+struct _Arrow {
   ArrowType type;
   real length;
   real width;
-} Arrow;
+};
 
-/* Can't include it earlier as it uses the Arrow def */
-#include "render.h"
-
-void arrow_draw(Renderer *renderer, ArrowType type,
+void arrow_draw(DiaRenderer *renderer, ArrowType type,
 		Point *to, Point *from,
 		real length, real width, real linewidth,
 		Color *fg_color, Color *bg_color);
 
 void
-calculate_arrow_point(Point *to, Point *from, Point *move,
-		      real length, real width, real linewidth);
+calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
+		      Point *move_arrow, Point *move_line,
+		      const real linewidth);
+
+/* Transforms 'start' to be at the back end of the arrow, and puts the
+ * tip of the arrow into 'arrowtip'.
+ */
+void arrow_transform_points(Arrow *arrow, Point *start, Point *to,
+                       int linewidth, Point *arrowtip);
 
 #endif /* ARROWS_H */
