@@ -20,11 +20,10 @@
 #ifndef BEZIER_CONN_H
 #define BEZIER_CONN_H
 
+#include "diatypes.h"
 #include "object.h"
 #include "boundingbox.h"
 
-typedef struct _BezierConn BezierConn;
-
 typedef enum {
   BEZ_CORNER_SYMMETRIC,
   BEZ_CORNER_SMOOTH,
@@ -46,10 +45,11 @@ struct _BezierConn {
 
 void bezierconn_update_data(BezierConn *bez);
 void bezierconn_update_boundingbox(BezierConn *bez);
-void bezierconn_simple_draw(BezierConn *bez, Renderer *renderer,
+void bezierconn_simple_draw(BezierConn *bez, DiaRenderer *renderer,
 			    real width);
-void bezierconn_draw_control_lines(BezierConn *bez, Renderer *renderer);
+void bezierconn_draw_control_lines(BezierConn *bez, DiaRenderer *renderer);
 void bezierconn_init(BezierConn *bez, int num_points);
+void bezierconn_set_points(BezierConn *poly, int num_points, BezPoint *points);
 void bezierconn_destroy(BezierConn *bez);
 void bezierconn_copy(BezierConn *from, BezierConn *to);
 void bezierconn_save(BezierConn *bez, ObjectNode obj_node);
