@@ -18,10 +18,9 @@
 #ifndef FOCUS_H
 #define FOCUS_H
 
+#include "diatypes.h"
 #include "object.h"
 
-typedef struct _Focus Focus;
-
 struct _Focus {
   Object *obj;
   int has_focus;
@@ -29,8 +28,8 @@ struct _Focus {
 
   /* return TRUE if modified object.
      Set change if object is changed. */
-  int (*key_event)(Focus *focus, guint keysym, char *str, int strlen,
-		   ObjectChange **change);
+  int (*key_event)(Focus *focus, guint keysym, const gchar *str, int strlen,
+                   ObjectChange **change);
 };
 
 void request_focus(Focus *focus);
