@@ -18,6 +18,7 @@
 #ifndef ORTH_CONN_H
 #define ORTH_CONN_H
 
+#include "diatypes.h"
 #include "object.h"
 #include "boundingbox.h"
 
@@ -30,8 +31,6 @@ typedef enum {
 
 #define HANDLE_MIDPOINT (HANDLE_CUSTOM1)
 
-typedef struct _OrthConn OrthConn;
-
 /* This is a subclass of Object used to help implementing objects
  * that connect points with orthogonal line-segments.
  */
@@ -58,7 +57,7 @@ struct _OrthConn {
 
 void orthconn_update_data(OrthConn *orth);
 void orthconn_update_boundingbox(OrthConn *orth);
-void orthconn_simple_draw(OrthConn *orth, Renderer *renderer,
+void orthconn_simple_draw(OrthConn *orth, DiaRenderer *renderer,
 			  real width);
 void orthconn_init(OrthConn *orth, Point *startpoint);
 void orthconn_destroy(OrthConn *orth);
