@@ -18,12 +18,12 @@
 #ifndef POLYSHAPE_H
 #define POLYSHAPE_H
 
+#include "diatypes.h"
 #include "object.h"
 #include "boundingbox.h"
 
 #define HANDLE_CORNER (HANDLE_CUSTOM1)
 
-typedef struct _PolyShape PolyShape;
 /* This is a subclass of Object used to help implementing objects
  * that form a polygon-like shape of line-segments.
  */
@@ -39,7 +39,7 @@ struct _PolyShape {
 
 void polyshape_update_data(PolyShape *poly);
 void polyshape_update_boundingbox(PolyShape *poly);
-void polyshape_simple_draw(PolyShape *poly, Renderer *renderer, real width);
+void polyshape_simple_draw(PolyShape *poly, DiaRenderer *renderer, real width);
 void polyshape_init(PolyShape *poly, int num_points);
 void polyshape_set_points(PolyShape *poly, int num_points, Point *points);
 void polyshape_destroy(PolyShape *poly);
