@@ -24,14 +24,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "neworth_conn.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
@@ -74,8 +73,8 @@ static void sadtarrow_move_handle(Sadtarrow *sadtarrow, Handle *handle,
 				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void sadtarrow_move(Sadtarrow *sadtarrow, Point *to);
 static void sadtarrow_select(Sadtarrow *sadtarrow, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void sadtarrow_draw(Sadtarrow *sadtarrow, DiaRenderer *renderer);
 static Object *sadtarrow_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -190,7 +189,7 @@ sadtarrow_distance_from(Sadtarrow *sadtarrow, Point *point)
 
 static void
 sadtarrow_select(Sadtarrow *sadtarrow, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   neworthconn_update_data(&sadtarrow->orth);
 }
@@ -215,19 +214,20 @@ sadtarrow_move(Sadtarrow *sadtarrow, Point *to)
   sadtarrow_update_data(sadtarrow);
 }
 
-static void draw_arrowhead(Renderer *renderer,
+static void draw_arrowhead(DiaRenderer *renderer,
 			   Point *end, Point *vect, Color *col);
-static void draw_dot(Renderer *renderer,
+static void draw_dot(DiaRenderer *renderer,
 		     Point *end, Point *vect, Color *col);
-static void draw_tunnel(Renderer *renderer,
+static void draw_tunnel(DiaRenderer *renderer,
 			     Point *end, Point *vect, Color *col);
 
 #define GBASE .45
 #define GMULT .55
 
 static void
-sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
+sadtarrow_draw(Sadtarrow *sadtarrow, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   NewOrthConn *orth = &sadtarrow->orth;
   Point *points;
   int n;
@@ -239,9 +239,9 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
   points = &orth->points[0];
   n = orth->numpoints;
   
-  renderer->ops->set_linewidth(renderer, ARROW_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, ARROW_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
   
   col = ARROW_COLOR;
   if (sadtarrow->autogray && 
@@ -281,8 +281,8 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
       
       if (rr < .01) {
 	/* too small corner : we do it a bit faster */
-	renderer->ops->draw_line(renderer,&m1,&M,&col);
-	renderer->ops->draw_line(renderer,&M,&m2,&col);
+	renderer_ops->draw_line(renderer,&m1,&M,&col);
+	renderer_ops->draw_line(renderer,&M,&m2,&col);
        } else {
          /* full rendering. We know len1 and len2 are nonzero. */
          v1.x = (M.x - X.x) / len1;
@@ -292,10 +292,10 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
        
          A.x = M.x - (v1.x * rr);
          A.y = M.y - (v1.y * rr);
-         renderer->ops->draw_line(renderer,&m1,&A,&col);
+         renderer_ops->draw_line(renderer,&m1,&A,&col);
          B.x = M.x + (v2.x * rr);
          B.y = M.y + (v2.y * rr);
-         renderer->ops->draw_line(renderer,&B,&m2,&col);
+         renderer_ops->draw_line(renderer,&B,&m2,&col);
          C.x = A.x + (v2.x * rr); /* (or B.x - v1.x*rr) */
          C.y = A.y + (v2.y * rr);
          
@@ -310,7 +310,12 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
 	 if ((ca > 1E-7) && (cb > 1E-7)) {
 	   cosa = vca.x / ca; sina = vca.y / ca;
 	   cosb = vcb.x / cb; sinb = vcb.y / cb;
-
+	   /* Seems that sometimes vca.x > ca etc (though it should be
+	    * impossible), so we make border cases here */
+	   if (cosa > 1.0) cosa = 1.0;
+	   if (cosa < -1.0) cosa = -1.0;
+	   if (cosb > 1.0) cosa = 1.0;
+	   if (cosb < -1.0) cosb = -1.0;
 	   /*cosg = ((M.x-X.x)*(Y.x-M.x) + (M.y-X.y)*(Y.y-M.y))/(len1*len2);*/
 	   sing = (-(M.x-X.x)*(Y.y-M.y) + (Y.x-M.x)*(M.y-X.y)) / (len1*len2);
 
@@ -318,7 +323,6 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
 	   if (sina < 0.0) alpha = -alpha;
 	   beta = acos(cosb) * 180.0 / M_PI ; 
 	   if (sinb < 0.0) beta = -beta;
-         
 	   /* we'll keep gamma in radians, since we're only interested in its
 	      sign */
 	   /* after all, we don't even need to compute cos(gamma), and
@@ -334,8 +338,8 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
 	     beta=alpha;
 	     alpha = tau;
 	   }
-	   
-	   renderer->ops->draw_arc(renderer,&C,rr*2,rr*2,alpha,beta,
+
+	   renderer_ops->draw_arc(renderer,&C,rr*2,rr*2,alpha,beta,
 				   &col);
 	 }
        }
@@ -371,7 +375,7 @@ sadtarrow_draw(Sadtarrow *sadtarrow, Renderer *renderer)
     break;
   }
 }
-static void draw_arrowhead(Renderer *renderer,
+static void draw_arrowhead(DiaRenderer *renderer,
 		       Point *end, Point *vect, Color *col)
 {
     arrow_draw(renderer, ARROW_HEAD_TYPE,
@@ -380,9 +384,10 @@ static void draw_arrowhead(Renderer *renderer,
 	       ARROW_LINE_WIDTH,
 	       col, &color_white);
 }
-static void draw_dot(Renderer *renderer,
+static void draw_dot(DiaRenderer *renderer,
 		     Point *end, Point *vect, Color *col)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point vv,vp,vt,pt;
   real vlen;
   vv = *end;
@@ -402,15 +407,16 @@ static void draw_dot(Renderer *renderer,
   point_scale(&vt,-ARROW_DOT_LOFFSET);
   point_add(&pt,&vt);
   
-  renderer->ops->set_fillstyle(renderer,FILLSTYLE_SOLID);
-  renderer->ops->fill_ellipse(renderer,&pt,
+  renderer_ops->set_fillstyle(renderer,FILLSTYLE_SOLID);
+  renderer_ops->fill_ellipse(renderer,&pt,
 			 ARROW_DOT_RADIUS,ARROW_DOT_RADIUS,
 			 col);
 }
 
-static void draw_tunnel(Renderer *renderer,
+static void draw_tunnel(DiaRenderer *renderer,
 			     Point *end, Point *vect, Color *col)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point vv,vp,vt1,vt2;
   BezPoint curve1[2];
   BezPoint curve2[2];
@@ -448,8 +454,8 @@ static void draw_tunnel(Renderer *renderer,
   point_add(&curve1[1].p3,&vt1);  point_add(&curve2[1].p3,&vt1); 
   point_sub(&curve1[1].p3,&vt2);  point_add(&curve2[1].p3,&vt2); 
 
-  renderer->ops->draw_bezier(renderer,curve1,2,col);
-  renderer->ops->draw_bezier(renderer,curve2,2,col);
+  renderer_ops->draw_bezier(renderer,curve1,2,col);
+  renderer_ops->draw_bezier(renderer,curve2,2,col);
 }
 
 
