@@ -48,8 +48,11 @@ extern ObjectType message_type;
 extern ObjectType component_type;
 extern ObjectType classicon_type;
 extern ObjectType state_type;
+extern ObjectType activity_type;
 extern ObjectType node_type;
 extern ObjectType branch_type;
+extern ObjectType fork_type;
+extern ObjectType state_term_type;
 
 DIA_PLUGIN_CHECK_INIT
 
@@ -79,33 +82,36 @@ dia_plugin_init(PluginInfo *info)
   object_register_type(&message_type);  
   object_register_type(&component_type);
   object_register_type(&classicon_type);
-  object_register_type(&state_type);    
+  object_register_type(&state_type);
+  object_register_type(&state_term_type);
+  object_register_type(&activity_type);
   object_register_type(&node_type);    
   object_register_type(&branch_type);    
-
+  object_register_type(&fork_type);
+   
   return DIA_PLUGIN_INIT_OK;
 }
 
 /* Warning, the following *must* be strictly ASCII characters (or fix the 
    following code for UTF-8 cleanliness */
 
-utfchar visible_char[] = { '+', '-', '#', ' ' };
+char visible_char[] = { '+', '-', '#', ' ' };
 
-utfchar *
+char *
 uml_get_attribute_string (UMLAttribute *attribute)
 {
   int len;
-  utfchar *str;
+  char *str;
 
   len = 1 + strlen (attribute->name) + strlen (attribute->type);
   if (attribute->name[0] && attribute->type[0]) {
     len += 2;
   }
-  if (attribute->value != NULL) {
+  if (attribute->value != NULL && attribute->value[0] != '\0') {
     len += 3 + strlen (attribute->value);
   }
   
-  str = g_malloc (sizeof (utfchar) * (len + 1));
+  str = g_malloc (sizeof (char) * (len + 1));
 
   str[0] = visible_char[(int) attribute->visibility];
   str[1] = 0;
@@ -115,7 +121,7 @@ uml_get_attribute_string (UMLAttribute *attribute)
     strcat (str, ": ");
   }
   strcat (str, attribute->type);
-  if (attribute->value != NULL) {
+  if (attribute->value != NULL && attribute->value[0] != '\0') {
     strcat (str, " = ");
     strcat (str, attribute->value);
   }
@@ -125,17 +131,17 @@ uml_get_attribute_string (UMLAttribute *attribute)
   return str;
 }
 
-utfchar *
+char *
 uml_get_operation_string (UMLOperation *operation)
 {
   int len;
-  utfchar *str;
+  char *str;
   GList *list;
   UMLParameter *param;
 
   /* Calculate length: */
   len = 1 + strlen (operation->name)  + 1;
-  if(operation->stereotype != NULL) {
+  if(operation->stereotype != NULL && operation->stereotype[0] != '\0') {
     len += 5 + strlen (operation->stereotype);
   }   
   
@@ -158,11 +164,14 @@ uml_get_operation_string (UMLOperation *operation)
 	len += 6;
 	break;	  
       }
-    len += strlen (param->name) + strlen (param->type);
-    if (param->type[0] && param->name[0]) {
-      len += 1;
+    len += strlen (param->name);
+    if (param->type != NULL) {
+      len += strlen (param->type);
+      if (param->type[0] && param->name[0]) {
+        len += 1;
+      }
     }
-    if (param->value != NULL) {
+    if (param->value != NULL && param->value[0] != '\0') {
       len += 1 + strlen (param->value);
     }
     
@@ -180,12 +189,12 @@ uml_get_operation_string (UMLOperation *operation)
   }
 
   /* generate string: */
-  str = g_malloc (sizeof (utfchar) * (len + 1));
+  str = g_malloc (sizeof (char) * (len + 1));
 
   str[0] = visible_char[(int) operation->visibility];
   str[1] = 0;
 
-  if(operation->stereotype != NULL) {
+  if(operation->stereotype != NULL && operation->stereotype[0] != '\0') {
     strcat(str, UML_STEREOTYPE_START);
     strcat(str, operation->stereotype);
     strcat(str, UML_STEREOTYPE_END);
@@ -215,12 +224,15 @@ uml_get_operation_string (UMLOperation *operation)
 	break;
       }
     strcat (str, param->name);
-    if (param->type[0] && param->name[0]) {
-      strcat (str, ":");
+
+    if (param->type != NULL) {
+      if (param->type[0] && param->name[0]) {
+        strcat (str, ":");
+      }
+      strcat (str, param->type);
     }
-    strcat (str, param->type);
     
-    if (param->value != NULL) {
+    if (param->value != NULL && param->value[0] != '\0') {
       strcat (str, "=");
       strcat (str, param->value);
     }
@@ -245,11 +257,11 @@ uml_get_operation_string (UMLOperation *operation)
   return str;
 }
 
-utfchar *
+char *
 uml_get_parameter_string (UMLParameter *param)
 {
   int len;
-  utfchar *str;
+  char *str;
 
   /* Calculate length: */
   len = strlen (param->name) + 1 + strlen (param->type);
@@ -274,7 +286,7 @@ uml_get_parameter_string (UMLParameter *param)
     }
 
   /* Generate string: */
-  str = g_malloc (sizeof (utfchar) * (len + 1));
+  str = g_malloc (sizeof (char) * (len + 1));
 
   strcpy(str, "");
 
@@ -307,11 +319,11 @@ uml_get_parameter_string (UMLParameter *param)
   return str;
 }
 
-utfchar *
+char *
 uml_get_formalparameter_string (UMLFormalParameter *parameter)
 {
   int len;
-  utfchar *str;
+  char *str;
 
   /* Calculate length: */
   len = strlen (parameter->name);
@@ -321,7 +333,7 @@ uml_get_formalparameter_string (UMLFormalParameter *parameter)
   }
 
   /* Generate string: */
-  str = g_malloc (sizeof (utfchar) * (len + 1));
+  str = g_malloc (sizeof (char) * (len + 1));
   strcpy (str, parameter->name);
   if (parameter->type != NULL) {
     strcat (str, ":");
@@ -347,6 +359,11 @@ uml_attribute_copy(UMLAttribute *attr)
   } else {
     newattr->value = NULL;
   }
+  if (attr->comment != NULL)
+    newattr->comment = g_strdup (attr->comment);
+  else 
+    newattr->comment = NULL;
+
   newattr->visibility = attr->visibility;
   newattr->abstract = attr->abstract;
   newattr->class_scope = attr->class_scope;
@@ -377,7 +394,9 @@ uml_operation_copy(UMLOperation *op)
   } else {
     newop->stereotype = NULL;
   }
-    
+  
+  newop->comment = g_strdup(op->comment);
+
   newop->visibility = op->visibility;
   newop->class_scope = op->class_scope;
   newop->inheritance_type = op->inheritance_type;
@@ -395,6 +414,8 @@ uml_operation_copy(UMLOperation *op)
     newparam = g_new0(UMLParameter, 1);
     newparam->name = g_strdup(param->name);
     newparam->type = g_strdup(param->type);
+    newparam->comment = g_strdup(param->comment);
+
     if (param->value != NULL)
       newparam->value = g_strdup(param->value);
     else
@@ -422,7 +443,7 @@ uml_formalparameter_copy(UMLFormalParameter *param)
   } else {
     newparam->type = NULL;
   }
-  
+
   return newparam;
 }
 
@@ -433,6 +454,9 @@ uml_attribute_destroy(UMLAttribute *attr)
   g_free(attr->type);
   if (attr->value != NULL)
     g_free(attr->value);
+  if (attr->comment != NULL)
+    g_free(attr->comment);
+
   g_free(attr);
 }
 
@@ -448,6 +472,8 @@ uml_operation_destroy(UMLOperation *op)
   if (op->stereotype != NULL)
     g_free(op->stereotype);
 
+  g_free(op->comment);
+
   list = op->parameters;
   while (list != NULL) {
     param = (UMLParameter *)list->data;
@@ -464,6 +490,8 @@ uml_parameter_destroy(UMLParameter *param)
   g_free(param->type);
   if (param->value != NULL) 
     g_free(param->value);
+  g_free(param->comment);
+
   g_free(param);
 }
 
@@ -485,6 +513,7 @@ uml_attribute_new(void)
   attr->name = g_strdup("");
   attr->type = g_strdup("");
   attr->value = NULL;
+  attr->comment = g_strdup("");
   attr->visibility = UML_PUBLIC;
   attr->abstract = FALSE;
   attr->class_scope = FALSE;
@@ -503,9 +532,10 @@ uml_operation_new(void)
   op->name = g_strdup("");
   op->type = NULL;
   op->stereotype = NULL;
+  op->comment = g_strdup("");
   op->visibility = UML_PUBLIC;
   op->class_scope = FALSE;
-  op->inheritance_type = UML_POLYMORPHIC;
+  op->inheritance_type = UML_LEAF;
   op->query = FALSE;
 
   op->parameters = NULL;
@@ -523,6 +553,7 @@ uml_parameter_new(void)
   param = g_new0(UMLParameter, 1);
   param->name = g_strdup("");
   param->type = g_strdup("");
+  param->comment = g_strdup("");
   param->value = NULL;
   param->kind = UML_UNDEF_KIND;
 
@@ -554,6 +585,8 @@ uml_attribute_write(AttributeNode attr_node, UMLAttribute *attr)
 		  attr->type);
   data_add_string(composite_add_attribute(composite, "value"),
 		  attr->value);
+  data_add_string(composite_add_attribute(composite, "comment"),
+		  attr->comment);
   data_add_enum(composite_add_attribute(composite, "visibility"),
 		attr->visibility);
   data_add_boolean(composite_add_attribute(composite, "abstract"),
@@ -581,6 +614,8 @@ uml_operation_write(AttributeNode attr_node, UMLOperation *op)
 		  op->type);
   data_add_enum(composite_add_attribute(composite, "visibility"),
 		op->visibility);
+  data_add_string(composite_add_attribute(composite, "comment"),
+		  op->comment);
   /* Backward compatibility */
   data_add_boolean(composite_add_attribute(composite, "abstract"),
 		   op->inheritance_type == UML_ABSTRACT);
@@ -605,6 +640,8 @@ uml_operation_write(AttributeNode attr_node, UMLOperation *op)
 		    param->type);
     data_add_string(composite_add_attribute(composite2, "value"),
 		    param->value);
+    data_add_string(composite_add_attribute(composite2, "comment"),
+		    param->comment);
     data_add_enum(composite_add_attribute(composite2, "kind"),
 		  param->kind);
     list = g_list_next(list);
@@ -647,6 +684,11 @@ uml_attribute_read(DataNode composite)
   if (attr_node != NULL)
     attr->value =  data_string( attribute_first_data(attr_node) );
   
+  attr->comment = NULL;
+  attr_node = composite_find_attribute(composite, "comment");
+  if (attr_node != NULL)
+    attr->comment =  data_string( attribute_first_data(attr_node) );
+
   attr->visibility = FALSE;
   attr_node = composite_find_attribute(composite, "visibility");
   if (attr_node != NULL)
@@ -695,6 +737,11 @@ uml_operation_read(DataNode composite)
   if (attr_node != NULL)
     op->stereotype = data_string( attribute_first_data(attr_node) );
 
+  op->comment = NULL;
+  attr_node = composite_find_attribute(composite, "comment");
+  if (attr_node != NULL)
+    op->comment = data_string( attribute_first_data(attr_node) );
+
   op->visibility = FALSE;
   attr_node = composite_find_attribute(composite, "visibility");
   if (attr_node != NULL)
@@ -742,6 +789,11 @@ uml_operation_read(DataNode composite)
     if (attr_node != NULL)
       param->value =  data_string( attribute_first_data(attr_node) );
     
+    param->comment = NULL;
+    attr_node = composite_find_attribute(composite2, "comment");
+    if (attr_node != NULL)
+      param->comment =  data_string( attribute_first_data(attr_node) );
+
     param->kind = UML_UNDEF_KIND;
     attr_node = composite_find_attribute(composite2, "kind");
     if (attr_node != NULL)
