@@ -28,16 +28,8 @@
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
 #include <glib.h>
 
-/* FIXME: not sure if this is already defined in Glib-1.2.x */
-#ifndef G_MODULE_EXPORT
-#define G_MODULE_EXPORT /* nothing is ok for *nix */
-#endif
-
 #include "sheet.h"
 #include "shape_info.h"
 #include "dia_dirs.h"
@@ -72,55 +64,45 @@ custom_object_load(gchar *filename, ObjectType **otype)
 
 static void load_shapes_from_tree(const gchar *directory)
 {
-  DIR *dp;
-  struct dirent *dirp;
-  struct stat statbuf;
+  GDir *dp;
+  const char *dentry;
 
-  dp = opendir(directory);
+  dp = g_dir_open(directory, 0, NULL);
   if (dp == NULL) {
     return;
   }
-  while ( (dirp = readdir(dp)) ) {
+  while ( (dentry = g_dir_read_name(dp)) ) {
     gchar *filename = g_strconcat(directory, G_DIR_SEPARATOR_S,
-				  dirp->d_name, NULL);
-    gchar *p;
+				  dentry, NULL);
+    const gchar *p;
 
-    if (!strcmp(dirp->d_name, ".") || !strcmp(dirp->d_name, "..")) {
-      g_free(filename);
-      continue;
-    }
-
-    if (stat(filename, &statbuf) < 0) {
-      g_free(filename);
-      continue;
-    }
-    if (S_ISDIR(statbuf.st_mode)) {
+    if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
       load_shapes_from_tree(filename);
       g_free(filename);
       continue;
     }
     /* if it's not a directory, then it must be a .shape file */
-    if (!S_ISREG(statbuf.st_mode) || (strlen(dirp->d_name) < 6)) {
+    if (   !g_file_test(filename, G_FILE_TEST_IS_REGULAR)
+        || (strlen(dentry) < 6)) {
       g_free(filename);
       continue;
     }
     
-
-    p = dirp->d_name + strlen(dirp->d_name) - 6;
+    p = dentry + strlen(dentry) - 6;
     if (0==strcmp(".shape",p)) {
       ObjectType *ot;
 
       if (!custom_object_load(filename, &ot)) {
-	g_warning("could not load shape file %s",filename);
+        g_warning("could not load shape file %s",filename);
       } else {
-	g_assert(ot); 
-	g_assert(ot->default_user_data);
-	object_register_type(ot);
+        g_assert(ot); 
+        g_assert(ot->default_user_data);
+        object_register_type(ot);
       }
     }
     g_free(filename);
   }
-  closedir(dp);
+  g_dir_close(dp);
 }
 
 
@@ -130,7 +112,7 @@ PluginInitResult
 dia_plugin_init(PluginInfo *info)
 {
   char *shape_path;
-  char *home_dir;
+  const char *home_dir;
 
   if (!dia_plugin_info_init(info, _("Custom"), _("Custom XML shapes loader"),
 			    NULL, NULL))
@@ -140,7 +122,7 @@ dia_plugin_init(PluginInfo *info)
   if (home_dir) {
     home_dir = dia_config_filename("shapes");
     load_shapes_from_tree(home_dir);
-    g_free(home_dir);
+    g_free((char *)home_dir);
   }
 
   shape_path = getenv("DIA_SHAPE_PATH");
